'''
The one parameter exponential family distributions used by GLM.
'''
# TODO: quasi, quasibinomial, quasipoisson
# see http://www.biostat.jhsph.edu/~qli/biostatistics_r_doc/library/stats/html/family.html
# for comparison to R, and McCullagh and Nelder

import numpy as np
from scipy import special
import links as L
import varfuncs as V
FLOAT_EPS = np.finfo(float).eps


class Family(object):
    """
    The parent class for one-parameter exponential families.

    Parameters
    ----------
    link : a link function instance
        Link is the linear transformation function.
        See the individual families for available links.
    variance : a variance function
        Measures the variance as a function of the mean probabilities.
        See the individual families for the default variance function.

    See Also
    --------
    :ref:`links`

    """
    # TODO: change these class attributes, use valid somewhere...
    valid = [-np.inf, np.inf]

    links = []

    def _setlink(self, link):
        """
        Helper method to set the link for a family.

        Raises a ValueError exception if the link is not available.  Note that
        the error message might not be that informative because it tells you
        that the link should be in the base class for the link function.

        See glm.GLM for a list of appropriate links for each family but note
        that not all of these are currently available.
        """
        # TODO: change the links class attribute in the families to hold
        # meaningful information instead of a list of links instances such as
        # [<statsmodels.family.links.Log object at 0x9a4240c>,
        #  <statsmodels.family.links.Power object at 0x9a423ec>,
        #  <statsmodels.family.links.Power object at 0x9a4236c>]
        # for Poisson...
        self._link = link
        if not isinstance(link, L.Link):
            raise TypeError("The input should be a valid Link object.")
        if hasattr(self, "links"):
            validlink = link in self.links
            validlink = max([isinstance(link, _) for _ in self.links])
            if not validlink:
                errmsg = "Invalid link for family, should be in %s. (got %s)"
                raise ValueError(errmsg % (repr(self.links), link))

    def _getlink(self):
        """
        Helper method to get the link for a family.
        """
        return self._link

    # link property for each family is a pointer to link instance
    link = property(_getlink, _setlink, doc="Link function for family")

    def __init__(self, link, variance):
        self.link = link()
        self.variance = variance

    def starting_mu(self, y):
        r"""
        Starting value for mu in the IRLS algorithm.

        Parameters
        ----------
        y : array
            The untransformed response variable.

        Returns
        -------
        mu_0 : array
            The first guess on the transformed response variable.

        Notes
        -----
        .. math::

           \mu_0 = (Y + \overline{Y})/2

        Notes
        -----
        Only the Binomial family takes a different initial value.
        """
        return (y + y.mean())/2.

    def weights(self, mu):
        r"""
        Weights for IRLS steps

        Parameters
        ----------
        mu : array-like
            The transformed mean response variable in the exponential family

        Returns
        -------
        w : array
            The weights for the IRLS steps

        Notes
        -----
        .. math::

           w = 1 / (g'(\mu)^2  * Var(\mu))
        """
        return 1. / (self.link.deriv(mu)**2 * self.variance(mu))

    def deviance(self, endog, mu, freq_weights=1., scale=1.):
        r"""
        The deviance function evaluated at (endog,mu,freq_weights,mu).

        Deviance is usually defined as twice the loglikelihood ratio.

        Parameters
        ----------
        endog : array-like
            The endogenous response variable
        mu : array-like
            The inverse of the link function at the linear predicted values.
        freq_weights : array-like
            1d array of frequency weights. The default is 1.
        scale : float, optional
            An optional scale argument. The default is 1.

        Returns
        -------
        Deviance : array
            The value of deviance function defined below.

        Notes
        -----
        Deviance is defined

        .. math::

           D = \sum_i (2 * freq\_weights_i * llf(Y_i, Y_i) - 2 *
               llf(Y_i, \mu_i)) / scale

        where y is the endogenous variable. The deviance functions are
        analytically defined for each family.
        """
        raise NotImplementedError

    def resid_dev(self, endog, mu, freq_weights=1., scale=1.):
        """
        The deviance residuals

        Parameters
        ----------
        endog : array
            The endogenous response variable
        mu : array
            The inverse of the link function at the linear predicted values.
        freq_weights : array-like
            1d array of frequency weights. The default is 1.
        scale : float, optional
            An optional argument to divide the residuals by scale. The default
            is 1.

        Returns
        -------
        Deviance residuals.

        Notes
        -----
        The deviance residuals are defined for each family.
        """
        raise NotImplementedError

    def fitted(self, lin_pred):
        """
        Fitted values based on linear predictors lin_pred.

        Parameters
        -----------
        lin_pred : array
            Values of the linear predictor of the model.
            dot(X,beta) in a classical linear model.

        Returns
        --------
        mu : array
            The mean response variables given by the inverse of the link
            function.
        """
        fits = self.link.inverse(lin_pred)
        return fits

    def predict(self, mu):
        """
        Linear predictors based on given mu values.

        Parameters
        ----------
        mu : array
            The mean response variables

        Returns
        -------
        lin_pred : array
            Linear predictors based on the mean response variables.  The value
            of the link function at the given mu.
        """
        return self.link(mu)

    def loglike(self, endog, mu, freq_weights=1., scale=1.):
        """
        The log-likelihood function in terms of the fitted mean response.

        Parameters
        ----------
        `endog` : array
            Usually the endogenous response variable.
        `mu` : array
            Usually but not always the fitted mean response variable.
        freq_weights : array-like
            1d array of frequency weights. The default is 1.
        scale : float
            The scale parameter. The default is 1.

        Returns
        -------
        llf : float
            The value of the loglikelihood evaluated at
            (endog,mu,freq_weights,scale) as defined below.
        Notes
        -----
        This is defined for each family.  endog and mu are not restricted to
        `endog` and `mu` respectively.  For instance, the deviance function
        calls both loglike(endog,endog) and loglike(endog,mu) to get the
        likelihood ratio.
        """
        raise NotImplementedError

    def resid_anscombe(self, endog, mu):
        """
        The Anscome residuals.

        See also
        --------
        statsmodels.families.family.Family docstring and the `resid_anscombe`
        for the individual families for more information.
        """
        raise NotImplementedError


class Poisson(Family):
    """
    Poisson exponential family.

    Parameters
    ----------
    link : a link instance, optional
        The default link for the Poisson family is the log link. Available
        links are log, identity, and sqrt. See statsmodels.family.links for
        more information.

    Attributes
    ----------
    Poisson.link : a link instance
        The link function of the Poisson instance.
    Poisson.variance : varfuncs instance
        `variance` is an instance of
        statsmodels.genmod.families.family.varfuncs.mu

    See also
    --------
    statsmodels.genmod.families.family.Family
    :ref:`links`

    """

    links = [L.log, L.identity, L.sqrt]
    variance = V.mu
    valid = [0, np.inf]
    safe_links = [L.Log, ]

    def __init__(self, link=L.log):
        self.variance = Poisson.variance
        self.link = link()

    def _clean(self, x):
        """
        Helper function to trim the data so that is in (0,inf)

        Notes
        -----
        The need for this function was discovered through usage and its
        possible that other families might need a check for validity of the
        domain.
        """
        return np.clip(x, FLOAT_EPS, np.inf)

    def resid_dev(self, endog, mu, scale=1.):
        r"""Poisson deviance residual

        Parameters
        ----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            Fitted mean response variable
        scale : float, optional
            An optional argument to divide the residuals by scale. The default
            is 1.

        Returns
        -------
        resid_dev : array
            Deviance residuals as defined below

        Notes
        -----
        .. math::

           resid\_dev_i = sign(Y_i - \mu_i) * \sqrt{2 *
                          (Y_i * \log(Y_i / \mu_i) - (Y_i - \mu_i))} / scale
        """
        endog_mu = self._clean(endog / mu)
        return (np.sign(endog - mu) *
                np.sqrt(2 * (endog * np.log(endog_mu) - (endog - mu))) / scale)

    def deviance(self, endog, mu, freq_weights=1., scale=1.):
        r'''
        Poisson deviance function

        Parameters
        ----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            Fitted mean response variable
        freq_weights : array-like
            1d array of frequency weights. The default is 1.
        scale : float, optional
            An optional scale argument. The default is 1.

        Returns
        -------
        deviance : float
            The deviance function at (endog,mu,freq_weights,scale) as defined
            below.

        Notes
        -----
        If a constant term is included it is defined as

        .. math::

           D = 2 * \sum_i (freq\_weights_i * Y_i * \log(Y_i / \mu_i))/ scale
        '''
        endog_mu = self._clean(endog / mu)
        return 2 * np.sum(endog * freq_weights * np.log(endog_mu)) / scale

    def loglike(self, endog, mu, freq_weights=1., scale=1.):
        r"""
        The log-likelihood function in terms of the fitted mean response.

        Parameters
        ----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            Fitted mean response variable
        freq_weights : array-like
            1d array of frequency weights. The default is 1.
        scale : float, optional
            The scale parameter, defaults to 1.

        Returns
        -------
        llf : float
            The value of the loglikelihood function evaluated at
            (endog,mu,freq_weights,scale) as defined below.

        Notes
        -----
        .. math::

           llf = scale * \sum_i freq\_weights_i * (Y_i * \log(\mu_i) - \mu_i -
                 \ln \Gamma(Y_i + 1))
        """
        loglike = np.sum(freq_weights * (endog * np.log(mu) - mu -
                         special.gammaln(endog + 1)))
        return scale * loglike

    def resid_anscombe(self, endog, mu):
        r"""
        Anscombe residuals for the Poisson exponential family distribution

        Parameters
        ----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            Fitted mean response variable

        Returns
        -------
        resid_anscombe : array
            The Anscome residuals for the Poisson family defined below

        Notes
        -----
        .. math::

           resid\_anscombe_i = (3/2) * (Y_i^{2/3} - \mu_i^{2/3}) / \mu_i^{1/6}
        """
        return (3 / 2.) * (endog**(2/3.) - mu**(2 / 3.)) / mu**(1 / 6.)

class QuasiPoisson(Family):
    """
    QuasiPoisson exponential family.

    Parameters
    ----------
    link : a link instance, optional
        The default link for the Poisson family is the log link. Available
        links are log, identity, and sqrt. See statsmodels.family.links for
        more information.

    Attributes
    ----------
    Poisson.link : a link instance
        The link function of the Poisson instance.
    Poisson.variance : varfuncs instance
        `variance` is an instance of
        statsmodels.genmod.families.family.varfuncs.mu

    See also
    --------
    statsmodels.genmod.families.family.Family
    :ref:`links`

    """

    links = [L.log, L.identity, L.sqrt]
    variance = V.mu
    valid = [0, np.inf]
    safe_links = [L.Log, ]

    def __init__(self, link=L.log):
        self.variance = Poisson.variance
        self.link = link()

    def _clean(self, x):
        """
        Helper function to trim the data so that is in (0,inf)

        Notes
        -----
        The need for this function was discovered through usage and its
        possible that other families might need a check for validity of the
        domain.
        """
        return np.clip(x, FLOAT_EPS, np.inf)

    def resid_dev(self, endog, mu, scale=1.):
        r"""Poisson deviance residual

        Parameters
        ----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            Fitted mean response variable
        scale : float, optional
            An optional argument to divide the residuals by scale. The default
            is 1.

        Returns
        -------
        resid_dev : array
            Deviance residuals as defined below

        Notes
        -----
        .. math::

           resid\_dev_i = sign(Y_i - \mu_i) * \sqrt{2 *
                          (Y_i * \log(Y_i / \mu_i) - (Y_i - \mu_i))} / scale
        """
        endog_mu = self._clean(endog / mu)
        return (np.sign(endog - mu) *
                np.sqrt(2 * (endog * np.log(endog_mu) - (endog - mu))) / scale)

    def deviance(self, endog, mu, freq_weights=1., scale=1.):
        r'''
        Poisson deviance function

        Parameters
        ----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            Fitted mean response variable
        freq_weights : array-like
            1d array of frequency weights. The default is 1.
        scale : float, optional
            An optional scale argument. The default is 1.

        Returns
        -------
        deviance : float
            The deviance function at (endog,mu,freq_weights,scale) as defined
            below.

        Notes
        -----
        If a constant term is included it is defined as

        .. math::

           D = 2 * \sum_i (freq\_weights_i * Y_i * \log(Y_i / \mu_i))/ scale
        '''
        endog_mu = self._clean(endog / mu)
        return 2 * np.sum(endog * freq_weights * np.log(endog_mu)) / scale

    def loglike(self, endog, mu, freq_weights=1., scale=1.):
        r"""
        The log-likelihood function in terms of the fitted mean response.

        Returns NaN for QuasiPoisson 

        Returns
        -------
        None: not applicable for QuasiPoisson
        """
        return np.nan

    def resid_anscombe(self, endog, mu):
        r"""
        Anscombe residuals for the Poisson exponential family distribution

        Parameters
        ----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            Fitted mean response variable

        Returns
        -------
        resid_anscombe : array
            The Anscome residuals for the Poisson family defined below

        Notes
        -----
        .. math::

           resid\_anscombe_i = (3/2) * (Y_i^{2/3} - \mu_i^{2/3}) / \mu_i^{1/6}
        """
        return (3 / 2.) * (endog**(2/3.) - mu**(2 / 3.)) / mu**(1 / 6.)

class Gaussian(Family):
    """
    Gaussian exponential family distribution.

    Parameters
    ----------
    link : a link instance, optional
        The default link for the Gaussian family is the identity link.
        Available links are log, identity, and inverse.
        See statsmodels.family.links for more information.

    Attributes
    ----------
    Gaussian.link : a link instance
        The link function of the Gaussian instance
    Gaussian.variance : varfunc instance
        `variance` is an instance of statsmodels.family.varfuncs.constant

    See also
    --------
    statsmodels.genmod.families.family.Family
    :ref:`links`

    """

    links = [L.log, L.identity, L.inverse_power]
    variance = V.constant
    safe_links = links

    def __init__(self, link=L.identity):
        self.variance = Gaussian.variance
        self.link = link()

    def resid_dev(self, endog, mu, scale=1.):
        r"""
        Gaussian deviance residuals

        Parameters
        -----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            Fitted mean response variable
        scale : float, optional
            An optional argument to divide the residuals by scale. The default
            is 1.

        Returns
        -------
        resid_dev : array
            Deviance residuals as defined below

        Notes
        --------
        .. math::

           resid\_dev_i = (Y_i - \mu_i) / \sqrt{Var(\mu_i)} / scale
        """

        return (endog - mu) / np.sqrt(self.variance(mu)) / scale

    def deviance(self, endog, mu, freq_weights=1., scale=1.):
        r"""
        Gaussian deviance function

        Parameters
        ----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            Fitted mean response variable
        freq_weights : array-like
            1d array of frequency weights. The default is 1.
        scale : float, optional
            An optional scale argument. The default is 1.

        Returns
        -------
        deviance : float
            The deviance function at (endog,mu,freq_weights,scale)
            as defined below.

        Notes
        --------
        .. math::

           D = \sum_i freq\_weights_i * (Y_i - \mu_i)^2 / scale
        """
        return np.sum((freq_weights * (endog - mu)**2)) / scale

    def loglike(self, endog, mu, freq_weights=1., scale=1.):
        r"""
        The log-likelihood in terms of the fitted mean response.

        Parameters
        ----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            Fitted mean response variable
        freq_weights : array-like
            1d array of frequency weights. The default is 1.
        scale : float, optional
            Scales the loglikelihood function. The default is 1.

        Returns
        -------
        llf : float
            The value of the loglikelihood function evaluated at
            (endog,mu,freq_weights,scale) as defined below.

        Notes
        -----
        If the link is the identity link function then the
        loglikelihood function is the same as the classical OLS model.

        .. math::

           llf = -nobs / 2 * (\log(SSR) + (1 + \log(2 \pi / nobs)))

        where

        .. math::
           SSR = \sum_i (Y_i - g^{-1}(\mu_i))^2

        If the links is not the identity link then the loglikelihood
        function is defined as

        .. math::

           llf = \sum_i freq\_weights_i * ((Y_i * \mu_i - \mu_i^2 / 2) / scale-
                 Y^2 / (2 * scale) - (1/2) * \log(2 * \pi * scale))
        """
        if isinstance(self.link, L.Power) and self.link.power == 1:
            # This is just the loglikelihood for classical OLS
            nobs2 = endog.shape[0] / 2.
            SSR = np.sum((endog-self.fitted(mu))**2, axis=0)
            llf = -np.log(SSR) * nobs2
            llf -= (1+np.log(np.pi/nobs2))*nobs2
            return llf
        else:
            return np.sum(freq_weights * ((endog * mu - mu**2/2)/scale -
                          endog**2/(2 * scale) - .5*np.log(2 * np.pi * scale)))

    def resid_anscombe(self, endog, mu):
        r"""
        The Anscombe residuals for the Gaussian exponential family distribution

        Parameters
        ----------
        endog : array
            Endogenous response variable
        mu : array
            Fitted mean response variable

        Returns
        -------
        resid_anscombe : array
            The Anscombe residuals for the Gaussian family defined below

        Notes
        --------
        .. math::

           resid\_anscombe_i = Y_i - \mu_i
        """
        return endog - mu


class Gamma(Family):
    """
    Gamma exponential family distribution.

    Parameters
    ----------
    link : a link instance, optional
        The default link for the Gamma family is the inverse link.
        Available links are log, identity, and inverse.
        See statsmodels.family.links for more information.

    Attributes
    ----------
    Gamma.link : a link instance
        The link function of the Gamma instance
    Gamma.variance : varfunc instance
        `variance` is an instance of statsmodels.family.varfuncs.mu_squared

    See also
    --------
    statsmodels.genmod.families.family.Family
    :ref:`links`

    """

    links = [L.log, L.identity, L.inverse_power]
    variance = V.mu_squared
    safe_links = [L.Log, ]

    def __init__(self, link=L.inverse_power):
        self.variance = Gamma.variance
        self.link = link()

    def _clean(self, x):
        """
        Helper function to trim the data so that is in (0,inf)

        Notes
        -----
        The need for this function was discovered through usage and its
        possible that other families might need a check for validity of the
        domain.
        """
        return np.clip(x, FLOAT_EPS, np.inf)

    def deviance(self, endog, mu, freq_weights=1., scale=1.):
        r"""
        Gamma deviance function

        Parameters
        -----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            Fitted mean response variable
        freq_weights : array-like
            1d array of frequency weights. The default is 1.
        scale : float, optional
            An optional scale argument. The default is 1.

        Returns
        -------
        deviance : float
            Deviance function as defined below

        Notes
        -----
        .. math::

           D = 2 * \sum_i freq\_weights_i * ((Y_i - \mu_i)/\mu_i - \log(Y_i /
               \mu_i))
        """
        endog_mu = self._clean(endog/mu)
        return 2*np.sum(freq_weights*((endog-mu)/mu-np.log(endog_mu)))

    def resid_dev(self, endog, mu, scale=1.):
        r"""
        Gamma deviance residuals

        Parameters
        -----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            Fitted mean response variable
        scale : float, optional
            An optional argument to divide the residuals by scale. The default
            is 1.

        Returns
        -------
        resid_dev : array
            Deviance residuals as defined below

        Notes
        -----
        .. math::

           resid\_dev_i = sign(Y_i - \mu_i) \sqrt{-2 *
                          (-(Y_i - \mu_i) / \mu_i + \log(Y_i / \mu_i))}
        """
        endog_mu = self._clean(endog / mu)
        return np.sign(endog - mu) * np.sqrt(-2 * (-(endog - mu)/mu +
                                                   np.log(endog_mu)))

    def loglike(self, endog, mu, freq_weights=1., scale=1.):
        r"""
        The log-likelihood function in terms of the fitted mean response.

        Parameters
        ----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            Fitted mean response variable
        freq_weights : array-like
            1d array of frequency weights. The default is 1.
        scale : float, optional
            The default is 1.

        Returns
        -------
        llf : float
            The value of the loglikelihood function evaluated at
            (endog,mu,freq_weights,scale) as defined below.

        Notes
        --------
        .. math::

           llf = -1 / scale * \sum_i *(Y_i / \mu_i+ \log(\mu_i)+
                 (scale -1) * \log(Y) + \log(scale) + scale *
                 \ln \Gamma(1 / scale))
        """
        return - 1./scale * np.sum((endog/mu + np.log(mu) + (scale - 1) *
                                    np.log(endog) + np.log(scale) + scale *
                                   special.gammaln(1./scale)) * freq_weights)

        # in Stata scale is set to equal 1 for reporting llf
        # in R it's the dispersion, though there is a loss of precision vs.
        # our results due to an assumed difference in implementation

    def resid_anscombe(self, endog, mu):
        r"""
        The Anscombe residuals for Gamma exponential family distribution

        Parameters
        ----------
        endog : array
            Endogenous response variable
        mu : array
            Fitted mean response variable

        Returns
        -------
        resid_anscombe : array
            The Anscombe residuals for the Gamma family defined below

        Notes
        -----
        .. math::

           resid\_anscombe_i = 3 * (Y_i^{1/3} - \mu_i^{1/3}) / \mu_i^{1/3}
        """
        return 3 * (endog**(1/3.) - mu**(1/3.)) / mu**(1/3.)


class Binomial(Family):
    """
    Binomial exponential family distribution.

    Parameters
    ----------
    link : a link instance, optional
        The default link for the Binomial family is the logit link.
        Available links are logit, probit, cauchy, log, and cloglog.
        See statsmodels.family.links for more information.

    Attributes
    ----------
    Binomial.link : a link instance
        The link function of the Binomial instance
    Binomial.variance : varfunc instance
        `variance` is an instance of statsmodels.family.varfuncs.binary

    See also
    --------
    statsmodels.genmod.families.family.Family
    :ref:`links`

    Notes
    -----
    endog for Binomial can be specified in one of three ways.

    """

    links = [L.logit, L.probit, L.cauchy, L.log, L.cloglog, L.identity]
    variance = V.binary  # this is not used below in an effort to include n

    # Other safe links, e.g. cloglog and probit are subclasses
    safe_links = [L.Logit, L.CDFLink]

    def __init__(self, link=L.logit):  # , n=1.):
        # TODO: it *should* work for a constant n>1 actually, if freq_weights
        # is equal to n
        self.n = 1
        # overwritten by initialize if needed but always used to initialize
        # variance since endog is assumed/forced to be (0,1)
        self.variance = V.Binomial(n=self.n)
        self.link = link()

    def starting_mu(self, y):
        """
        The starting values for the IRLS algorithm for the Binomial family.
        A good choice for the binomial family is :math:`\mu_0 = (Y_i + 0.5)/2`
        """
        return (y + .5)/2

    def initialize(self, endog, freq_weights):
        '''
        Initialize the response variable.

        Parameters
        ----------
        endog : array
            Endogenous response variable

        Returns
        --------
        If `endog` is binary, returns `endog`

        If `endog` is a 2d array, then the input is assumed to be in the format
        (successes, failures) and
        successes/(success + failures) is returned.  And n is set to
        successes + failures.
        '''
        # if not np.all(np.asarray(freq_weights) == 1):
        #     self.variance = V.Binomial(n=freq_weights)
        if (endog.ndim > 1 and endog.shape[1] > 1):
            y = endog[:, 0]
            # overwrite self.freq_weights for deviance below
            self.n = endog.sum(1)
            return y*1./self.n, self.n
        else:
            return endog, np.ones(endog.shape[0])

    def deviance(self, endog, mu, freq_weights=1, scale=1., axis=None):
        r'''
        Deviance function for either Bernoulli or Binomial data.

        Parameters
        ----------
        endog : array-like
            Endogenous response variable (already transformed to a probability
            if appropriate).
        mu : array
            Fitted mean response variable
        freq_weights : array-like
            1d array of frequency weights. The default is 1.
        scale : float, optional
            An optional scale argument. The default is 1.

        Returns
        --------
        deviance : float
            The deviance function as defined below

        Notes
        -----
        If the endogenous variable is binary:

        .. math::

           D = -2 * \sum_i freq\_weights * (I_{1,i} * \log(\mu_i) + I_{0,i} *
               \log(1 - \mu_i))

        where :math:`I_{1,i}` is an indicator function that evalueates to 1 if
        :math:`Y_i = 1`. and :math:`I_{0,i}` is an indicator function that
        evaluates to 1 if :math:`Y_i = 0`.

        If the model is ninomial:

        .. math::

           D = 2 * \sum_i freq\_weights * (\log(Y_i / \mu_i) + (n_i - Y_i) *
               \log((n_i - Y_i) / n_i - \mu_i))

        where :math:`Y_i` and :math:`n` are as defined in Binomial.initialize.
        '''
        if np.shape(self.n) == () and self.n == 1:
            one = np.equal(endog, 1)
            return -2 * np.sum((one * np.log(mu + 1e-200) + (1-one) *
                               np.log(1 - mu + 1e-200)) * freq_weights, axis=axis)

        else:
            return 2 * np.sum(self.n * freq_weights *
                              (endog * np.log(endog/mu + 1e-200) +
                               (1 - endog) * np.log((1 - endog) /
                               (1 - mu) + 1e-200)), axis=axis)

    def resid_dev(self, endog, mu, scale=1.):
        r"""
        Binomial deviance residuals

        Parameters
        -----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            Fitted mean response variable
        scale : float, optional
            An optional argument to divide the residuals by scale. The default
            is 1.

        Returns
        -------
        resid_dev : array
            Deviance residuals as defined below

        Notes
        -----
        If the endogenous variable is binary:

        .. math::

           resid\_dev_i = sign(Y_i - \mu_i) * \sqrt{-2 *
                          \log(I_{1,i} * \mu_i + I_{0,i} * (1 - \mu_i))}

        where :math:`I_{1,i}` is an indicator function that evalueates to 1 if
        :math:`Y_i = 1`. and :math:`I_{0,i}` is an indicator function that
        evaluates to 1 if :math:`Y_i = 0`.

        If the endogenous variable is binomial:

        .. math::

           resid\_dev_i = sign(Y_i - \mu_i) \sqrt{2 * n_i *
                          (Y_i * \log(Y_i / \mu_i) + (1 - Y_i) *
                          \log(1 - Y_i)/(1 - \mu_i))}

        where :math:`Y_i` and :math:`n` are as defined in Binomial.initialize.
        """

        mu = self.link._clean(mu)
        if np.shape(self.n) == () and self.n == 1:
            one = np.equal(endog, 1)
            return np.sign(endog-mu)*np.sqrt(-2 *
                                             np.log(one * mu + (1 - one) *
                                                    (1 - mu)))/scale
        else:
            return (np.sign(endog - mu) *
                    np.sqrt(2 * self.n *
                            (endog * np.log(endog/mu + 1e-200) +
                             (1 - endog) * np.log((1 - endog)/(1 - mu) + 1e-200)))/scale)

    def loglike(self, endog, mu, freq_weights=1, scale=1.):
        r"""
        The log-likelihood function in terms of the fitted mean response.

        Parameters
        ----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            Fitted mean response variable
        freq_weights : array-like
            1d array of frequency weights. The default is 1.
        scale : float, optional
            Not used for the Binomial GLM.

        Returns
        -------
        llf : float
            The value of the loglikelihood function evaluated at
            (endog,mu,freq_weights,scale) as defined below.

        Notes
        --------
        If the endogenous variable is binary:

        .. math::

         llf = scale * \sum_i (y_i * \log(\mu_i/(1-\mu_i)) + \log(1-\mu_i)) *
               freq\_weights_i

        If the endogenous variable is binomial:

        .. math::

           llf = scale * \sum_i freq\_weights_i * (\ln \Gamma(n+1) -
                 \ln \Gamma(y_i + 1) - \ln \Gamma(n_i - y_i +1) + y_i *
                 \log(\mu_i / (1 - \mu_i)) + n * \log(1 - \mu_i))

        where :math:`y_i = Y_i * n_i` with :math:`Y_i` and :math:`n_i` as
        defined in Binomial initialize.  This simply makes :math:`y_i` the
        original number of successes.
        """

        if np.shape(self.n) == () and self.n == 1:
            return scale * np.sum((endog * np.log(mu/(1 - mu) + 1e-200) +
                                   np.log(1 - mu)) * freq_weights)
        else:
            y = endog * self.n  # convert back to successes
            return scale * np.sum((special.gammaln(self.n + 1) -
                                   special.gammaln(y + 1) -
                                   special.gammaln(self.n - y + 1) + y *
                                   np.log(mu/(1 - mu)) + self.n *
                                   np.log(1 - mu)) * freq_weights)

    def resid_anscombe(self, endog, mu):
        '''
        The Anscombe residuals

        Parameters
        ----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            Fitted mean response variable

        Returns
        -------
        resid_anscombe : array
            The Anscombe residuals as defined below.

        Notes
        -----
        sqrt(n)*(cox_snell(endog)-cox_snell(mu))/(mu**(1/6.)*(1-mu)**(1/6.))

        where cox_snell is defined as
        cox_snell(x) = betainc(2/3., 2/3., x)*betainc(2/3.,2/3.)
        where betainc is the incomplete beta function

        The name 'cox_snell' is idiosyncratic and is simply used for
        convenience following the approach suggested in Cox and Snell (1968).
        Further note that
        cox_snell(x) = x**(2/3.)/(2/3.)*hyp2f1(2/3.,1/3.,5/3.,x)
        where hyp2f1 is the hypergeometric 2f1 function.  The Anscombe
        residuals are sometimes defined in the literature using the
        hyp2f1 formulation.  Both betainc and hyp2f1 can be found in scipy.

        References
        ----------
        Anscombe, FJ. (1953) "Contribution to the discussion of H. Hotelling's
            paper." Journal of the Royal Statistical Society B. 15, 229-30.

        Cox, DR and Snell, EJ. (1968) "A General Definition of Residuals."
            Journal of the Royal Statistical Society B. 30, 248-75.

        '''
        cox_snell = lambda x: (special.betainc(2/3., 2/3., x)
                               * special.beta(2/3., 2/3.))
        return np.sqrt(self.n) * ((cox_snell(endog) - cox_snell(mu)) /
                                  (mu**(1/6.) * (1 - mu)**(1/6.)))


class InverseGaussian(Family):
    """
    InverseGaussian exponential family.

    Parameters
    ----------
    link : a link instance, optional
        The default link for the inverse Gaussian family is the
        inverse squared link.
        Available links are inverse_squared, inverse, log, and identity.
        See statsmodels.family.links for more information.

    Attributes
    ----------
    InverseGaussian.link : a link instance
        The link function of the inverse Gaussian instance
    InverseGaussian.variance : varfunc instance
        `variance` is an instance of statsmodels.family.varfuncs.mu_cubed

    See also
    --------
    statsmodels.genmod.families.family.Family
    :ref:`links`

    Notes
    -----
    The inverse Guassian distribution is sometimes referred to in the
    literature as the Wald distribution.

    """

    links = [L.inverse_squared, L.inverse_power, L.identity, L.log]
    variance = V.mu_cubed
    safe_links = [L.inverse_squared, L.Log, ]

    def __init__(self, link=L.inverse_squared):
        self.variance = InverseGaussian.variance
        self.link = link()

    def resid_dev(self, endog, mu, scale=1.):
        r"""
        Returns the deviance residuals for the inverse Gaussian family.

        Parameters
        -----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            Fitted mean response variable
        freq_weights : array-like
            1d array of frequency weights. The default is 1.
        scale : float, optional
            An optional argument to divide the residuals by scale. The default
            is 1.

        Returns
        -------
        resid_dev : array
            Deviance residuals as defined below

        Notes
        -----
        .. math::

           resid\_dev_i = sign(Y_i - \mu_i) * 
                          \sqrt {(Y_i - \mu_i)^2 / (Y_i * \mu_i^2)} / scale
        """
        return np.sign(endog-mu) * np.sqrt((endog-mu)**2/(endog*mu**2))/scale

    def deviance(self, endog, mu, freq_weights=1., scale=1.):
        r"""
        Inverse Gaussian deviance function

        Parameters
        -----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            Fitted mean response variable
        freq_weights : array-like
            1d array of frequency weights. The default is 1.
        scale : float, optional
            An optional scale argument. The default is 1.

        Returns
        -------
        deviance : float
            Deviance function as defined below

        Notes
        -----
        .. math::

           D = \sum_i freq\_weights_i * ((Y_i - \mu_i)^2 / (Y_i *\mu_i^2)) /
               scale
        """
        return np.sum(freq_weights*(endog-mu)**2/(endog*mu**2))/scale

    def loglike(self, endog, mu, freq_weights=1., scale=1.):
        r"""
        The log-likelihood function in terms of the fitted mean response.

        Parameters
        ----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            Fitted mean response variable
        freq_weights : array-like
            1d array of frequency weights. The default is 1.
        scale : float, optional
            The default is 1.

        Returns
        -------
        llf : float
            The value of the loglikelihood function evaluated at
            (endog,mu,freq_weights,scale) as defined below.

        Notes
        -----
        .. math::

           llf = -1/2 * \sum_i freq\_weights_i * ((Y_i - \mu_i)^2 / (Y_i *
                 \mu_i * scale) + \log(scale * Y_i^3) + \log(2 * \pi))
        """
        return -.5 * np.sum(((endog - mu)**2/(endog * mu**2 * scale) +
                             np.log(scale * endog**3) + np.log(2 * np.pi)) *
                            freq_weights)

    def resid_anscombe(self, endog, mu):
        r"""
        The Anscombe residuals for the inverse Gaussian distribution

        Parameters
        ----------
        endog : array
            Endogenous response variable
        mu : array
            Fitted mean response variable

        Returns
        -------
        resid_anscombe : array
            The Anscombe residuals for the inverse Gaussian distribution  as
            defined below

        Notes
        -----
        .. math::

           resid\_anscombe_i = \log(Y_i / \mu_i) / \sqrt{\mu_i}
        """
        return np.log(endog / mu) / np.sqrt(mu)


class NegativeBinomial(Family):
    """
    Negative Binomial exponential family.

    Parameters
    ----------
    link : a link instance, optional
        The default link for the negative binomial family is the log link.
        Available links are log, cloglog, identity, nbinom and power.
        See statsmodels.family.links for more information.
    alpha : float, optional
        The ancillary parameter for the negative binomial distribution.
        For now `alpha` is assumed to be nonstochastic.  The default value
        is 1.  Permissible values are usually assumed to be between .01 and 2.


    Attributes
    ----------
    NegativeBinomial.link : a link instance
        The link function of the negative binomial instance
    NegativeBinomial.variance : varfunc instance
        `variance` is an instance of statsmodels.family.varfuncs.nbinom

    See also
    --------
    statsmodels.genmod.families.family.Family
    :ref:`links`

    Notes
    -----
    Power link functions are not yet supported.

    """
    links = [L.log, L.cloglog, L.identity, L.nbinom, L.Power]
    # TODO: add the ability to use the power links with an if test
    # similar to below
    variance = V.nbinom
    safe_links = [L.Log, ]

    def __init__(self, link=L.log, alpha=1.):
        self.alpha = 1. * alpha  # make it at least float
        self.variance = V.NegativeBinomial(alpha=self.alpha)
        if isinstance(link, L.NegativeBinomial):
            self.link = link(alpha=self.alpha)
        else:
            self.link = link()

    def _clean(self, x):
        """
        Helper function to trim the data so that is in (0,inf)

        Notes
        -----
        The need for this function was discovered through usage and its
        possible that other families might need a check for validity of the
        domain.
        """
        return np.clip(x, FLOAT_EPS, np.inf)

    def deviance(self, endog, mu, freq_weights=1., scale=1.):
        r"""
        Returns the value of the deviance function.

        Parameters
        -----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            Fitted mean response variable
        freq_weights : array-like
            1d array of frequency weights. The default is 1.
        scale : float, optional
            An optional scale argument. The default is 1.

        Returns
        -------
        deviance : float
            Deviance function as defined below

        Notes
        -----
        :math:`D = \sum_i piecewise_i` where :math:`piecewise_i` is defined as:

        If :math:`Y_{i} = 0`:

        :math:`piecewise_i = 2* \log(1 + \alpha * \mu_i) / \alpha`

        If :math:`Y_{i} > 0`:

        :math:`piecewise_i = 2 * Y_i * \log(Y_i / \mu_i) - (2 / \alpha) *
            (1 + \alpha * Y_i) * \ln(1 + \alpha * Y_i) / (1 + \alpha * \mu_i)`
        """
        iszero = np.equal(endog, 0)
        notzero = 1 - iszero
        endog_mu = self._clean(endog/mu)
        tmp = iszero * 2 * np.log(1 + self.alpha * mu) / self.alpha
        tmp += notzero * (2 * endog * np.log(endog_mu) - 2 / self.alpha *
                          (1 + self.alpha * endog) *
                          np.log((1 + self.alpha * endog) /
                                 (1 + self.alpha * mu)))
        return np.sum(freq_weights * tmp) / scale

    def resid_dev(self, endog, mu, scale=1.):
        r"""
        Negative Binomial Deviance Residual

        Parameters
        ----------
        endog : array-like
            `endog` is the response variable
        mu : array-like
            `mu` is the fitted value of the model
        scale : float, optional
            An optional argument to divide the residuals by scale. The default
            is 1.

        Returns
        --------
        resid_dev : array
            The array of deviance residuals

        Notes
        -----
        :math:`resid\_dev_i = sign(Y_i-\mu_i) * \sqrt{piecewise_i}`

        where :math:`piecewise_i` is defined as

        If :math:`Y_i = 0`:

        :math:`piecewise_i = 2 * \log(1 + \alpha * \mu_i)/ \alpha`

        If :math:`Y_i > 0`:

        :math:`piecewise_i = 2 * Y_i * \log(Y_i / \mu_i) - (2 / \alpha) *
        (1 + \alpha * Y_i) * \log((1 + \alpha * Y_i) / (1 + \alpha * \mu_i))`
        """
        iszero = np.equal(endog, 0)
        notzero = 1 - iszero
        endog_mu = self._clean(endog / mu)
        tmp = iszero * 2 * np.log(1 + self.alpha * mu) / self.alpha
        tmp += notzero * (2 * endog * np.log(endog_mu) - 2 / self.alpha *
                          (1 + self.alpha * endog) *
                          np.log((1 + self.alpha * endog) /
                                 (1 + self.alpha * mu)))
        return np.sign(endog - mu) * np.sqrt(tmp) / scale

    def loglike(self, endog, mu, freq_weights=1., scale=1.):
        r"""
        The log-likelihood function in terms of the fitted mean response.

        Parameters
        ----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            The fitted mean response values
        freq_weights : array-like
            1d array of frequency weights. The default is 1.
        scale : float
            The scale parameter. The default is 1.

        Returns
        -------
        llf : float
            The value of the loglikelihood function evaluated at
            (endog,mu,freq_weights,scale) as defined below.

        Notes
        -----
        Defined as:

        .. math::

           llf = \sum_i freq\_weights_i * (Y_i * \log{(\alpha * e^{\eta_i} /
                 (1 + \alpha * e^{\eta_i}))} - \log{(1 + \alpha * e^{\eta_i})}/
                 \alpha + Constant)

        where :math:`Constant` is defined as:

        .. math::

           Constant = \ln \Gamma{(Y_i + 1/ \alpha )} - \ln \Gamma(Y_i + 1) -
                      \ln \Gamma{(1/ \alpha )}
        """
        lin_pred = self._link(mu)
        constant = (special.gammaln(endog + 1 / self.alpha) -
                    special.gammaln(endog+1)-special.gammaln(1/self.alpha))
        exp_lin_pred = np.exp(lin_pred)
        return np.sum((endog * np.log(self.alpha * exp_lin_pred /
                                      (1 + self.alpha * exp_lin_pred)) -
                      np.log(1 + self.alpha * exp_lin_pred) /
                      self.alpha + constant) * freq_weights)

    def resid_anscombe(self, endog, mu):
        """
        The Anscombe residuals for the negative binomial family

        Parameters
        ----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            Fitted mean response variable

        Returns
        -------
        resid_anscombe : array
            The Anscombe residuals as defined below.

        Notes
        -----
        `resid_anscombe` = (hyp2f1(-alpha*endog)-hyp2f1(-alpha*mu)+\
                1.5*(endog**(2/3.)-mu**(2/3.)))/(mu+alpha*mu**2)**(1/6.)

        where hyp2f1 is the hypergeometric 2f1 function parameterized as
        hyp2f1(x) = hyp2f1(2/3.,1/3.,5/3.,x)
        """

        hyp2f1 = lambda x : special.hyp2f1(2 / 3., 1 / 3., 5 / 3., x)
        return ((hyp2f1(-self.alpha * endog) - hyp2f1(-self.alpha * mu) +
                 1.5 * ( endog**(2 / 3.) - mu**(2 / 3.))) /
                (mu + self.alpha * mu**2)**(1 / 6.))


class Tweedie(Family):
    """
    Tweedie family.

    Parameters
    ----------
    link : a link instance, optional
        The default link for the Tweedie family is the log link when the
        link_power is 0. Otherwise, the power link is default.
        Available links are log and Power.
    var_power : float, optional
        The variance power.
    link_power : float, optional
        The link power.

    Attributes
    ----------
    Tweedie.link : a link instance
        The link function of the Tweedie instance
    Tweedie.variance : varfunc instance
        `variance` is an instance of statsmodels.family.varfuncs.Power
    Tweedie.link_power : float
        The power of the link function, or 0 if its a log link.
    Tweedie.var_power : float
        The power of the variance function.

    See also
    --------
    statsmodels.genmod.families.family.Family
    :ref:`links`

    Notes
    -----
    Logliklihood function not implemented because of the complexity of
    calculating an infinite series of summations. The variance power can be
    estimated using the `estimate_tweedie_power` function that is part of the
    `GLM` class.
    """
    links = [L.log, L.Power]
    variance = V.Power
    safe_links = [L.log, L.Power]

    def __init__(self, link=None, var_power=1., link_power=0):
        self.var_power = var_power
        self.link_power = link_power
        self.variance = V.Power(power=var_power * 1.)
        if link_power != 0 and not ((link is L.Power) or (link is None)):
            msg = 'link_power of {} not supported specified link'
            msg = msg.format(link_power)
            raise ValueError(msg)
        if (link_power == 0) and ((link is None) or (link is L.Log)):
            self.link = L.log()
        elif link_power != 0:
            self.link = L.Power(power=link_power * 1.)
        else:
            self.link = link()

    def _clean(self, x):
        """
        Helper function to trim the data so that is in (0,inf)

        Notes
        -----
        The need for this function was discovered through usage and its
        possible that other families might need a check for validity of the
        domain.
        """
        return np.clip(x, 0, np.inf)

    def deviance(self, endog, mu, freq_weights=1., scale=1.):
        r"""
        Returns the value of the deviance function.

        Parameters
        -----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            Fitted mean response variable
        freq_weights : array-like
            1d array of frequency weights. The default is 1.
        scale : float, optional
            An optional scale argument. The default is 1.

        Returns
        -------
        deviance : float
            Deviance function as defined below

        Notes
        -----
        When :math:`p = 1`,

        .. math::

            resid\_dev_i = \mu

        when :math:`endog = 0` and

        .. math::

            resid\_dev_i = endog * \log(endog / \mu) + (\mu - endog)

        otherwise.

        When :math:`p = 2`,

        .. math::

            resid\_dev_i =  (endog - \mu) / \mu - \log(endog / \mu)

        For all other p,

        .. math::

            resid\_dev_i = endog ^{2 - p} / ((1 - p) * (2 - p)) -
                           endog * \mu ^{1 - p} / (1 - p) + \mu ^{2 - p} /
                           (2 - p)

        Once :math:`resid\_dev_i` is calculated, then calculate deviance as

        .. math::

            D = \sum{2 * freq\_weights * resid\_dev_i}
        """
        p = self.var_power
        if p == 1:
            dev = np.where(endog == 0,
                           mu,
                           endog * np.log(endog / mu) + (mu - endog))
        elif p == 2:
            endog1 = np.clip(endog, FLOAT_EPS, np.inf)
            dev = ((endog - mu) / mu) - np.log(endog1 / mu)
        else:
            dev = (endog ** (2 - p) / ((1 - p) * (2 - p)) -
                   endog * mu ** (1-p) / (1 - p) + mu ** (2 - p) / (2 - p))
        return np.sum(2 * freq_weights * dev)

    def resid_dev(self, endog, mu, scale=1.):
        r"""
        Tweedie Deviance Residual

        Parameters
        ----------
        endog : array-like
            `endog` is the response variable
        mu : array-like
            `mu` is the fitted value of the model
        scale : float, optional
            An optional argument to divide the residuals by scale. The default
            is 1.

        Returns
        --------
        resid_dev : array
            The array of deviance residuals

        Notes
        -----
        When :math:`p = 1`,

        .. math::

            resid\_dev_i = \mu

        when :math:`endog = 0` and

        .. math::

            resid\_dev_i = endog * \log(endog / \mu) + (\mu - endog)

        otherwise.

        When :math:`p = 2`,

        .. math::

            resid\_dev_i =  (endog - \mu) / \mu - \log(endog / \mu)

        For all other p,

        .. math::

            resid\_dev_i = endog ^{2 - p} / ((1 - p) * (2 - p)) -
                           endog * \mu ^{1 - p} / (1 - p) + \mu ^{2 - p} /
                           (2 - p)
        """
        p = self.var_power
        if p == 1:
            dev = np.where(endog == 0,
                           mu,
                           endog * np.log(endog / mu) + (mu - endog))
        elif p == 2:
            endog1 = np.clip(endog, FLOAT_EPS, np.inf)
            dev = ((endog - mu) / mu) - np.log(endog1 / mu)
        else:
            dev = (endog ** (2 - p) / ((1 - p) * (2 - p)) -
                   endog * mu ** (1-p) / (1 - p) + mu ** (2 - p) / (2 - p))
        return np.sign(endog - mu) * np.sqrt(2 * dev)

    def loglike(self, endog, mu, freq_weights=1., scale=1.):
        r"""
        The log-likelihood function in terms of the fitted mean response.

        Parameters
        ----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            The fitted mean response values
        freq_weights : array-like
            1d array of frequency weights. The default is 1.
        scale : float
            The scale parameter. The default is 1.

        Returns
        -------
        llf : float
            The value of the loglikelihood function evaluated at
            (endog,mu,freq_weights,scale) as defined below.

        Notes
        -----
        This is not implemented because of the complexity of calculating an
        infinite series of sums.
        """
        return np.nan

    def resid_anscombe(self, endog, mu):
        """
        The Anscombe residuals for the Tweedie family

        Parameters
        ----------
        endog : array-like
            Endogenous response variable
        mu : array-like
            Fitted mean response variable

        Returns
        -------
        resid_anscombe : array
            The Anscombe residuals as defined below.

        Notes
        -----
        When :math:`p = 3`, then

        .. math::

            resid\_anscombe_i = (\log(endog) - \log(\mu)) / \sqrt{mu}

        Otherwise,

        .. math::

            c = (3 - p) / 3

        .. math::

            resid\_anscombe_i = (1 / c) * (endog ^ c - \mu ^ c) / \mu ^{p / 6}
        """
        if self.var_power == 3:
            return (np.log(endog) - np.log(mu)) / np.sqrt(mu)
        else:
            c = (3. - self.var_power) / 3.
            return ((1. / c) * (endog ** c - mu ** c) /
                    mu ** (self.var_power / 6.))



import pysal
import os.path
import struct
import pysal.core.FileIO as FileIO
from pysal.weights import W
from warnings import warn

__author__ = "Myunghwa Hwang <mhwang4@gmail.com>"
__all__ = ["Wk1IO"]


class Wk1IO(FileIO.FileIO):
    """
    MATLAB wk1read.m and wk1write.m that were written by Brian M. Bourgault in 10/22/93

    Opens, reads, and writes weights file objects in Lotus Wk1 format.

    Lotus Wk1 file is used in Dr. LeSage's MATLAB Econometrics library.

    A Wk1 file holds a spatial weights object in a full matrix form
    without any row and column headers.
    The maximum number of columns supported in a Wk1 file is 256.
    Wk1 starts the row (column) number from 0 and
    uses little endian binary endcoding.
    In PySAL, when the number of observations is n,
    it is assumed that each cell of a n\*n(=m) matrix either is a blank or
    have a number.

    The internal structure of a Wk1 file written by PySAL is as follows:
    [BOF][DIM][CPI][CAL][CMODE][CORD][SPLIT][SYNC][CURS][WIN]
    [HCOL][MRG][LBL][CELL_1]...[CELL_m][EOF]
    where [CELL_k] equals to [DTYPE][DLEN][DFORMAT][CINDEX][CVALUE].
    The parts between [BOF] and [CELL_1] are variable according to the software
    program used to write a wk1 file. While reading a wk1 file,
    PySAL ignores them.
    Each part of this structure is detailed below.

 .. table:: Lotus WK1 fields

   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |Part         |Description          |Data Type                |Length |Value                        |
   +=============+=====================+=========================+=======+=============================+
   |[BOF]        |Begining of field    |unsigned character       |6      |0,0,2,0,6,4                  |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |[DIM]        |Matrix dimension                                                                     |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |  [DIMDTYPE] |Type of dim. rec     |unsigned short           |2      |6                            |
   |  [DIMLEN]   |Length of dim. rec   |unsigned short           |2      |8                            |
   |  [DIMVAL]   |Value of dim. rec    |unsigned short           |8      |0,0,n,n                      |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |[CPI]        |CPI                                                                                  |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |  [CPITYPE]  |Type of cpi rec      |unsigned short           |2      |150                          |
   |  [CPILEN]   |Length of cpi rec    |unsigned short           |2      |6                            |
   |  [CPIVAL]   |Value of cpi rec     |unsigned char            |6      |0,0,0,0,0,0                  |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |[CAL]        |calcount                                                                             |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |  [CALTYPE]  |Type of calcount rec |unsigned short           |2      |47                           |
   |  [CALLEN]   |Length calcount rec  |unsigned short           |2      |1                            |
   |  [CALVAL]   |Value of calcount rec|unsigned char            |1      |0                            |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |[CMODE]      |calmode                                                                              |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |  [CMODETYP] |Type of calmode rec  |unsigned short           |2      |2                            |
   |  [CMODELEN] |Length of calmode rec|unsigned short           |2      |1                            |
   |  [CMODEVAL] |Value of calmode rec |signed char              |1      |0                            |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |[CORD]       |calorder                                                                             |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |  [CORDTYPE] |Type of calorder rec |unsigned short           |2      |3                            |
   |  [CORDLEN]  |Length calorder rec  |unsigned short           |2      |1                            |
   |  [CORDVAL]  |Value of calorder rec|signed char              |1      |0                            |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |[SPLIT]      |split                                                                                |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |  [SPLTYPE]  |Type of split rec    |unsigned short           |2      |4                            |
   |  [SPLLEN]   |Length of split rec  |unsigned short           |2      |1                            |
   |  [SPLVAL]   |Value of split rec   |signed char              |1      |0                            |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |[SYNC]       |sync                                                                                 |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |  [SYNCTYP]  |Type of sync rec     |unsigned short           |2      |5                            |
   |  [SYNCLEN]  |Length of sync rec   |unsigned short           |2      |1                            |
   |  [SYNCVAL]  |Value of sync rec    |singed char              |1      |0                            |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |[CURS]       |cursor                                                                               |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |  [CURSTYP]  |Type of cursor rec   |unsigned short           |2      |49                           |
   |  [CURSLEN]  |Length of cursor rec |unsigned short           |2      |1                            |
   |  [CURSVAL]  |Value of cursor rec  |signed char              |1      |1                            |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |[WIN]        |window                                                                               |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |  [WINTYPE]  |Type of window rec   |unsigned short           |2      |7                            |
   |  [WINLEN]   |Length of window rec |unsigned short           |2      |32                           |
   |  [WINVAL1]  |Value 1 of window rec|unsigned short           |4      |0,0                          |
   |  [WINVAL2]  |Value 2 of window rec|signed char              |2      |113,0                        |
   |  [WINVAL3]  |Value 3 of window rec|unsigned short           |26     |10,n,n,0,0,0,0,0,0,0,0,72,0  |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |[HCOL]       |hidcol                                                                               |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |  [HCOLTYP]  |Type of hidcol rec   |unsigned short           |2      |100                          |
   |  [HCOLLEN]  |Length of hidcol rec |unsigned short           |2      |32                           |
   |  [HCOLVAL]  |Value of hidcol rec  |signed char              |32     |0*32                         |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |[MRG]        |margins                                                                              |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |  [MRGTYPE]  |Type of margins rec  |unsigned short           |2      |40                           |
   |  [MRGLEN]   |Length of margins rec|unsigned short           |2      |10                           |
   |  [MRGVAL]   |Value of margins rec |unsigned short           |10     |4,76,66,2,2                  |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |[LBL]        |labels                                                                               |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |  [LBLTYPE]  |Type of labels rec   |unsigned short           |2      |41                           |
   |  [LBLLEN]   |Length of labels rec |unsigned short           |2      |1                            |
   |  [LBLVAL]   |Value of labels rec  |char                     |1      |'                            |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |[CELL_k]                                                                                           |
   +-------------+---------------------+-------------------------+-------+-----------------------------+
   |  [DTYPE]    |Type of cell data    |unsigned short           |2      |[DTYPE][0]==0: end of file   |
   |             |                     |                         |       |          ==14: number       |
   |             |                     |                         |       |          ==16: formula      |
   |             |                     |                         |       |          ==13: integer      |
   |             |                     |                         |       |          ==11: nrange       |
   |             |                     |                         |       |          ==else: unknown    |
   |  [DLEN]     |Length of cell data  |unsigned short           |2      |                             |
   |  [DFORMAT]  |Format of cell data  |not sure                 |1      |                             |
   |  [CINDEX]   |Row, column of cell  |unsigned short           |4      |                             |
   |  [CVALUE]   |Value of cell        |double, [DTYPE][0]==14   |8      |                             |
   |             |                     |formula,[DTYPE][0]==16   |8 +    |[DTYPE][1] - 13              |
   |             |                     |integer,[DTYPE][0]==13   |2      |                             |
   |             |                     |nrange, [DTYPE][0]==11   |24     |                             |
   |             |                     |else,   [DTYPE][0]==else |       |[DTYPE][1]                   |
   |  [EOF]      |End of file          |unsigned short           |4      |1,0,0,0                      |
   +-------------+---------------------+-------------------------+-------+-----------------------------+


    """

    FORMATS = ['wk1']
    MODES = ['r', 'w']

    def __init__(self, *args, **kwargs):
        self._varName = 'Unknown'
        FileIO.FileIO.__init__(self, *args, **kwargs)
        self.file = open(self.dataPath, self.mode + 'b')

    def _set_varName(self, val):
        if issubclass(type(val), basestring):
            self._varName = val

    def _get_varName(self):
        return self._varName
    varName = property(fget=_get_varName, fset=_set_varName)

    def read(self, n=-1):
        self._complain_ifclosed(self.closed)
        return self._read()

    def seek(self, pos):
        if pos == 0:
            self.file.seek(0)
            self.pos = 0

    def _read(self):
        """
        Reads Lotus Wk1 file

        Returns
        -------
        A pysal.weights.weights.W object

        Examples
        --------

        Type 'dir(w)' at the interpreter to see what methods are supported.
        Open a Lotus Wk1 file and read it into a pysal weights object

        >>> w = pysal.open(pysal.examples.get_path('spat-sym-us.wk1'),'r').read()

        Get the number of observations from the header

        >>> w.n
        46

        Get the mean number of neighbors

        >>> w.mean_neighbors
        4.0869565217391308

        Get neighbor distances for a single observation

        >>> w[1]
        {25: 1.0, 3: 1.0, 28: 1.0, 39: 1.0}

        """
        if self.pos > 0:
            raise StopIteration

        bof = struct.unpack('<6B', self.file.read(6))
        if bof != (0, 0, 2, 0, 6, 4):
            raise ValueError('The header of your file is wrong!')

        neighbors = {}
        weights = {}
        dtype, dlen = struct.unpack('<2H', self.file.read(4))
        while(dtype != 1):
            if dtype in [13, 14, 16]:
                self.file.read(1)
                row, column = struct.unpack('2H', self.file.read(4))
                format, length = '<d', 8
                if dtype == 13:
                    format, length = '<h', 2
                value = float(struct.unpack(format, self.file.read(length))[0])
                if value > 0:
                    ngh = neighbors.setdefault(row, [])
                    ngh.append(column)
                    wgt = weights.setdefault(row, [])
                    wgt.append(value)
                if dtype == 16:
                    self.file.read(dlen - 13)
            elif dtype == 11:
                self.file.read(24)
            else:
                self.file.read(dlen)
            dtype, dlen = struct.unpack('<2H', self.file.read(4))

        self.pos += 1
        return W(neighbors, weights)

    def write(self, obj):
        """

        Parameters
        ----------
        .write(weightsObject)
        accepts a weights object

        Returns
        ------

        a Lotus wk1 file
        write a weights object to the opened wk1 file.

        Examples
        --------

        >>> import tempfile, pysal, os
        >>> testfile = pysal.open(pysal.examples.get_path('spat-sym-us.wk1'),'r')
        >>> w = testfile.read()

        Create a temporary file for this example

        >>> f = tempfile.NamedTemporaryFile(suffix='.wk1')

        Reassign to new var

        >>> fname = f.name

        Close the temporary named file

        >>> f.close()

        Open the new file in write mode

        >>> o = pysal.open(fname,'w')

        Write the Weights object into the open file

        >>> o.write(w)
        >>> o.close()

        Read in the newly created text file

        >>> wnew =  pysal.open(fname,'r').read()

        Compare values from old to new

        >>> wnew.pct_nonzero == w.pct_nonzero
        True

        Clean up temporary file created for this example

        >>> os.remove(fname)

        """
        self._complain_ifclosed(self.closed)
        if issubclass(type(obj), W):
            f = self.file
            n = obj.n
            if n > 256:
                raise ValueError('WK1 file format supports only up to 256 observations.')
            pack = struct.pack
            f.write(pack('<6B', 0, 0, 2, 0, 6, 4))
            f.write(pack('<6H', 6, 8, 0, 0, n, n))
            f.write(pack('<2H6B', 150, 6, 0, 0, 0, 0, 0, 0))
            f.write(pack('<2H1B', 47, 1, 0))
            f.write(pack('<2H1b', 2, 1, 0))
            f.write(pack('<2H1b', 3, 1, 0))
            f.write(pack('<2H1b', 4, 1, 0))
            f.write(pack('<2H1b', 5, 1, 0))
            f.write(pack('<2H1b', 49, 1, 1))
            f.write(pack('<4H2b13H', 7, 32, 0, 0, 113, 0, 10,
                         n, n, 0, 0, 0, 0, 0, 0, 0, 0, 72, 0))
            hidcol = tuple(['<2H32b', 100, 32] + [0] * 32)
            f.write(pack(*hidcol))
            f.write(pack('<7H', 40, 10, 4, 76, 66, 2, 2))
            f.write(pack('<2H1c', 41, 1, "'".encode()))

            id2i = obj.id2i
            for i, w_i in enumerate(obj):
                row = [0.0] * n
                for k in w_i[1]:
                    row[id2i[k]] = w_i[1][k]
                for c, v in enumerate(row):
                    cell = tuple(['<2H1b2H1d', 14, 13, 113, i, c, v])
                    f.write(pack(*cell))
            f.write(pack('<4B', 1, 0, 0, 0))
            self.pos += 1

        else:
            raise TypeError("Expected a pysal weights object, got: %s" % (
                type(obj)))

    def close(self):
        self.file.close()
        FileIO.FileIO.close(self)





"""
sphere: Tools for working with spherical geometry.

Author(s):
    Charles R Schmidt schmidtc@gmail.com
    Luc Anselin luc.anselin@asu.edu
    Xun Li xun.li@asu.edu

"""

__author__ = "Charles R Schmidt <schmidtc@gmail.com>, Luc Anselin <luc.anselin@asu.edu, Xun Li <xun.li@asu.edu"

import math
import random
import numpy
import scipy.spatial
import scipy.constants
from scipy.spatial.distance import euclidean
from math import pi, cos, sin, asin

__all__ = ['RADIUS_EARTH_KM', 'RADIUS_EARTH_MILES', 'arcdist', 'arcdist2linear', 'brute_knn', 'fast_knn', 'fast_threshold', 'linear2arcdist', 'toLngLat', 'toXYZ', 'lonlat','harcdist','geointerpolate','geogrid']


RADIUS_EARTH_KM = 6371.0
RADIUS_EARTH_MILES = (
    RADIUS_EARTH_KM * scipy.constants.kilo) / scipy.constants.mile


def arcdist(pt0, pt1, radius=RADIUS_EARTH_KM):
    """
    Parameters
    ----------
    pt0 : point
        assumed to be in form (lng,lat)
    pt1 : point
        assumed to be in form (lng,lat)
    radius : radius of the sphere
        defaults to Earth's radius

        Source: http://nssdc.gsfc.nasa.gov/planetary/factsheet/earthfact.html

    Returns
    -------
    The arc distance between pt0 and pt1 using supplied radius

    Examples
    --------
    >>> pt0 = (0,0)
    >>> pt1 = (180,0)
    >>> d = arcdist(pt0,pt1,RADIUS_EARTH_MILES)
    >>> d == math.pi*RADIUS_EARTH_MILES
    True
    """
    return linear2arcdist(euclidean(toXYZ(pt0), toXYZ(pt1)), radius)


def arcdist2linear(arc_dist, radius=RADIUS_EARTH_KM):
    """
    Convert an arc distance (spherical earth) to a linear distance (R3) in the unit sphere.

    Examples
    --------
    >>> pt0 = (0,0)
    >>> pt1 = (180,0)
    >>> d = arcdist(pt0,pt1,RADIUS_EARTH_MILES)
    >>> d == math.pi*RADIUS_EARTH_MILES
    True
    >>> arcdist2linear(d,RADIUS_EARTH_MILES)
    2.0
    """
    c = 2 * math.pi * radius
    d = (2 - (2 * math.cos(math.radians((arc_dist * 360.0) / c)))) ** (0.5)
    return d


def linear2arcdist(linear_dist, radius=RADIUS_EARTH_KM):
    """
    Convert a linear distance in the unit sphere (R3) to an arc distance based on supplied radius

    Examples
    --------
    >>> pt0 = (0,0)
    >>> pt1 = (180,0)
    >>> d = arcdist(pt0,pt1,RADIUS_EARTH_MILES)
    >>> d == linear2arcdist(2.0, radius = RADIUS_EARTH_MILES)
    True
    """
    if linear_dist == float('inf'):
        return float('inf')
    elif linear_dist > 2.0:
        raise ValueError("linear_dist, must not exceed the diameter of the unit sphere, 2.0")
    c = 2 * math.pi * radius
    a2 = linear_dist ** 2
    theta = math.degrees(math.acos((2 - a2) / (2.)))
    d = (theta * c) / 360.0
    return d


def toXYZ(pt):
    """
    Parameters
    ----------
    pt0 : point
        assumed to be in form (lng,lat)
    pt1 : point
        assumed to be in form (lng,lat)

    Returns
    -------
    x, y, z
    """
    phi, theta = map(math.radians, pt)
    phi, theta = phi + pi, theta + (pi / 2)
    x = 1 * sin(theta) * cos(phi)
    y = 1 * sin(theta) * sin(phi)
    z = 1 * cos(theta)
    return x, y, z


def toLngLat(xyz):
    x, y, z = xyz
    if z == -1 or z == 1:
        phi = 0
    else:
        phi = math.atan2(y, x)
        if phi > 0:
            phi = phi - math.pi
        elif phi < 0:
            phi = phi + math.pi
    theta = math.acos(z) - (math.pi / 2)
    return phi, theta


def brute_knn(pts, k, mode='arc'):
    """
    valid modes are ['arc','xrz']
    """
    n = len(pts)
    full = numpy.zeros((n, n))
    for i in xrange(n):
        for j in xrange(i + 1, n):
            if mode == 'arc':
                lng0, lat0 = pts[i]
                lng1, lat1 = pts[j]
                dist = arcdist(pts[i], pts[j], radius=RADIUS_EARTH_KM)
            elif mode == 'xyz':
                dist = euclidean(pts[i], pts[j])
            full[i, j] = dist
            full[j, i] = dist
    w = {}
    for i in xrange(n):
        w[i] = full[i].argsort()[1:k + 1].tolist()
    return w


def fast_knn(pts, k, return_dist=False):
    """
    Computes k nearest neighbors on a sphere.

    Parameters
    ----------
    pts :  list of x,y pairs
    k   :  int
        Number of points to query
    return_dist : bool
        Return distances in the 'wd' container object

    Returns
    -------
    wn  :  list
        list of neighbors
    wd  : list
        list of neighbor distances (optional)

    """
    pts = numpy.array(pts)
    kd = scipy.spatial.KDTree(pts)
    d, w = kd.query(pts, k + 1)
    w = w[:, 1:]
    wn = {}
    for i in xrange(len(pts)):
        wn[i] = w[i].tolist()
    if return_dist:
        d = d[:, 1:]
        wd = {}
        for i in xrange(len(pts)):
            wd[i] = [linear2arcdist(x,
                                    radius=RADIUS_EARTH_MILES) for x in d[i].tolist()]
        return wn, wd
    return wn


def fast_threshold(pts, dist, radius=RADIUS_EARTH_KM):
    d = arcdist2linear(dist, radius)
    kd = scipy.spatial.KDTree(pts)
    r = kd.query_ball_tree(kd, d)
    wd = {}
    for i in xrange(len(pts)):
        l = r[i]
        l.remove(i)
        wd[i] = l
    return wd


########### new functions

def lonlat(pointslist):
    """
    Converts point order from lat-lon tuples to lon-lat (x,y) tuples

    Parameters
    ----------

    pointslist : list of lat-lon tuples (Note, has to be a list, even for one point)

    Returns
    -------

    newpts      : list with tuples of points in lon-lat order

    Example
    -------
    >>> points = [(41.981417, -87.893517), (41.980396, -87.776787), (41.980906, -87.696450)]
    >>> newpoints = lonlat(points)
    >>> newpoints
    [(-87.893517, 41.981417), (-87.776787, 41.980396), (-87.69645, 41.980906)]

    """
    newpts = [(i[1],i[0]) for i in pointslist]
    return newpts

def haversine(x):
    """
    Computes the haversine formula

    Parameters
    ----------
    x    : angle in radians

    Returns
    -------
         : square of sine of half the radian (the haversine formula)

    Example
    -------
    >>> haversine(math.pi)     # is 180 in radians, hence sin of 90 = 1
    1.0

    """

    x = math.sin(x/2)
    return x*x

# Lambda functions

# degree to radian conversion
d2r = lambda x: x * math.pi / 180.0

# radian to degree conversion
r2d = lambda x: x * 180.0 / math.pi

def radangle(p0,p1):
    """
    Radian angle between two points on a sphere in lon-lat (x,y)

    Parameters
    ----------
    p0    : first point as a lon,lat tuple
    p1    : second point as a lon,lat tuple

    Returns
    -------
    d     : radian angle in radians

    Example
    -------
    >>> p0 = (-87.893517, 41.981417)
    >>> p1 = (-87.519295, 41.657498)
    >>> radangle(p0,p1)
    0.007460167953189258

    Note
    ----
    Uses haversine formula, function haversine and degree to radian
    conversion lambda function d2r

    """
    x0, y0 = d2r(p0[0]),d2r(p0[1])
    x1, y1 = d2r(p1[0]),d2r(p1[1])
    d = 2.0 * math.asin(math.sqrt(haversine(y1 - y0) +
                        math.cos(y0) * math.cos(y1)*haversine(x1 - x0)))
    return d

def harcdist(p0,p1,lonx=True,radius=RADIUS_EARTH_KM):
    """
    Alternative arc distance function, uses haversine formula

    Parameters
    ----------
    p0       : first point as a tuple in decimal degrees
    p1       : second point as a tuple in decimal degrees
    lonx     : boolean to assess the order of the coordinates,
               for lon,lat (default) = True, for lat,lon = False
    radius   : radius of the earth at the equator as a sphere
               default: RADIUS_EARTH_KM (6371.0 km)
               options: RADIUS_EARTH_MILES (3959.0 miles)
                        None (for result in radians)

    Returns
    -------
    d        : distance in units specified, km, miles or radians (for None)

    Example
    -------
    >>> p0 = (-87.893517, 41.981417)
    >>> p1 = (-87.519295, 41.657498)
    >>> harcdist(p0,p1)
    47.52873002976876
    >>> harcdist(p0,p1,radius=None)
    0.007460167953189258

    Note
    ----
    Uses radangle function to compute radian angle

    """
    if not(lonx):
        p = lonlat([p0,p1])
        p0 = p[0]
        p1 = p[1]

    d = radangle(p0,p1)
    if radius is not None:
        d = d*radius
    return d

def geointerpolate(p0,p1,t,lonx=True):
    """
    Finds a point on a sphere along the great circle distance between two points
    on a sphere
    also known as a way point in great circle navigation

    Parameters
    ----------
    p0       : first point as a tuple in decimal degrees
    p1       : second point as a tuple in decimal degrees
    t        : proportion along great circle distance between p0 and p1
               e.g., t=0.5 would find the mid-point
    lonx     : boolean to assess the order of the coordinates,
               for lon,lat (default) = True, for lat,lon = False

    Returns
    -------
    x,y      : tuple in decimal degrees of lon-lat (default) or lat-lon,
               depending on setting of lonx; in other words, the same
               order is used as for the input

    Example
    -------
    >>> p0 = (-87.893517, 41.981417)
    >>> p1 = (-87.519295, 41.657498)
    >>> geointerpolate(p0,p1,0.1)          # using lon-lat
    (-87.85592403438788, 41.949079912574796)
    >>> p3 = (41.981417, -87.893517)
    >>> p4 = (41.657498, -87.519295)
    >>> geointerpolate(p3,p4,0.1,lonx=False)   # using lat-lon
    (41.949079912574796, -87.85592403438788)

    """

    if not(lonx):
        p = lonlat([p0,p1])
        p0 = p[0]
        p1 = p[1]

    d = radangle(p0,p1)
    k = 1.0 / math.sin(d)
    t = t*d
    A = math.sin(d-t) * k
    B = math.sin(t) * k

    x0, y0 = d2r(p0[0]),d2r(p0[1])
    x1, y1 = d2r(p1[0]),d2r(p1[1])

    x = A * math.cos(y0) * math.cos(x0) + B * math.cos(y1) * math.cos(x1)
    y = A * math.cos(y0) * math.sin(x0) + B * math.cos(y1) * math.sin(x1)
    z = A * math.sin(y0) + B * math.sin(y1)

    newpx = r2d(math.atan2(y, x))
    newpy = r2d(math.atan2(z, math.sqrt(x*x + y*y)))
    if not(lonx):
        return newpy,newpx
    return newpx,newpy

def geogrid(pup,pdown,k,lonx=True):
    """
    Computes a k+1 by k+1 set of grid points for a bounding box in lat-lon
    uses geointerpolate

    Parameters
    ----------
    pup     : tuple with lat-lon or lon-lat for upper left corner of bounding box
    pdown   : tuple with lat-lon or lon-lat for lower right corner of bounding box
    k       : number of grid cells (grid points will be one more)
    lonx    : boolean to assess the order of the coordinates,
              for lon,lat (default) = True, for lat,lon = False

    Returns
    -------
    grid    : list of tuples with lat-lon or lon-lat for grid points, row by row,
              starting with the top row and moving to the bottom; coordinate tuples
              are returned in same order as input

    Example
    -------
    >>> pup = (42.023768,-87.946389)    # Arlington Heights IL
    >>> pdown = (41.644415,-87.524102)  # Hammond, IN
    >>> geogrid(pup,pdown,3,lonx=False)
    [(42.023768, -87.946389), (42.02393997819538, -87.80562679358316), (42.02393997819538, -87.66486420641684), (42.023768, -87.524102), (41.897317, -87.94638900000001), (41.8974888973743, -87.80562679296166), (41.8974888973743, -87.66486420703835), (41.897317, -87.524102), (41.770866000000005, -87.94638900000001), (41.77103781320412, -87.80562679234043), (41.77103781320412, -87.66486420765956), (41.770866000000005, -87.524102), (41.644415, -87.946389), (41.64458672568646, -87.80562679171955), (41.64458672568646, -87.66486420828045), (41.644415, -87.524102)]

    """
    if lonx:
        corners = [pup,pdown]
    else:
        corners = lonlat([pup,pdown])
    tpoints = [float(i)/k for i in range(k)[1:]]
    leftcorners = [corners[0],(corners[0][0],corners[1][1])]
    rightcorners = [(corners[1][0],corners[0][1]),corners[1]]
    leftside = [leftcorners[0]]
    rightside = [rightcorners[0]]
    for t in tpoints:
        newpl = geointerpolate(leftcorners[0],leftcorners[1],t)
        leftside.append(newpl)
        newpr = geointerpolate(rightcorners[0],rightcorners[1],t)
        rightside.append(newpr)
    leftside.append(leftcorners[1])
    rightside.append(rightcorners[1])

    grid = []
    for i in range(len(leftside)):
        grid.append(leftside[i])
        for t in tpoints:
            newp = geointerpolate(leftside[i],rightside[i],t)
            grid.append(newp)
        grid.append(rightside[i])
    if not(lonx):
        grid = lonlat(grid)
    return grid



"""
A module for computational geometry.
"""
from shapes import *
from standalone import *
from locators import *
from rtree import *
from kdtree import *
from sphere import *
#why don't we import collection?




"""pyrtree Unittest."""
from pysal.cg import RTree, Rect
import unittest


class Pyrtree_Tester(unittest.TestCase):
    """setup class for unit tests."""
    def setUp(self):
        k = 10
        w = 20
        objects = {}
        id = 0
        for i in range(k):
            mn_y = i * w
            mx_y = mn_y + w
            for j in range(k):
                mn_x = j * w
                mx_x = mn_x + w
                objects[id] = Rect(mn_x, mn_y, mx_x, mx_y)
                id += 1
        self.objects = objects

    def test_rtree(self):
        t = RTree()
        for object in self.objects:
            t.insert(object, self.objects[object])
        self.assertEqual(len(self.objects), 100)

        qr = Rect(5, 5, 25, 25)

        # find objects with mbrs intersecting with qr
        res = [r.leaf_obj() for r in t.query_rect(qr) if r.is_leaf()]
        self.assertEqual(len(res), 4)
        res.sort()
        self.assertEqual(res, [0, 1, 10, 11])

        # vertices are shared by all coincident rectangles
        res = [r.leaf_obj(
        ) for r in t.query_point((20.0, 20.0)) if r.is_leaf()]
        self.assertEqual(len(res), 4)

        res = [r.leaf_obj() for r in t.query_point((21, 20)) if r.is_leaf()]
        self.assertEqual(len(res), 2)

        # single internal point
        res = [r.leaf_obj() for r in t.query_point((21, 21)) if r.is_leaf()]
        self.assertEqual(len(res), 1)

        # single external point
        res = [r.leaf_obj() for r in t.query_point((-12, 21)) if r.is_leaf()]
        self.assertEqual(len(res), 0)

        qr = Rect(5, 6, 65, 7)

        res = [r.leaf_obj() for r in t.query_rect((qr)) if r.is_leaf()]
        self.assertEqual(len(res), 4)


suite = unittest.TestSuite()
test_classes = [Pyrtree_Tester]
for i in test_classes:
    a = unittest.TestLoader().loadTestsFromTestCase(i)
    suite.addTest(a)

if __name__ == '__main__':
    runner = unittest.TextTestRunner()
    runner.run(suite)



import numpy as np
import numpy.linalg as la
from scipy import sparse as sp
from scipy.sparse import linalg as spla
from pysal.spreg.utils import spdot, spmultiply
from family import Binomial, Poisson

def _compute_betas(y, x):
    """
    compute MLE coefficients using iwls routine

    Methods: p189, Iteratively (Re)weighted Least Squares (IWLS),
    Fotheringham, A. S., Brunsdon, C., & Charlton, M. (2002).
    Geographically weighted regression: the analysis of spatially varying relationships.
    """
    xT = x.T
    xtx = spdot(xT, x)
    xtx_inv = la.inv(xtx)
    xtx_inv = sp.csr_matrix(xtx_inv)
    xTy = spdot(xT, y, array_out=False)
    betas = spdot(xtx_inv, xTy)
    return betas

def _compute_betas_gwr(y, x, wi):
    """
    compute MLE coefficients using iwls routine

    Methods: p189, Iteratively (Re)weighted Least Squares (IWLS),
    Fotheringham, A. S., Brunsdon, C., & Charlton, M. (2002).
    Geographically weighted regression: the analysis of spatially varying relationships.
    """
    xT = (x * wi).T
    xtx = np.dot(xT, x)
    xtx_inv = la.inv(xtx)
    xtx_inv_xt = np.dot(xtx_inv, xT)
    betas = np.dot(xtx_inv_xt, y)
    return betas, xtx_inv_xt

def iwls(y, x, family, offset, y_fix,
    ini_betas=None, tol=1.0e-8, max_iter=200, wi=None):
    """
    Iteratively re-weighted least squares estimation routine
    """
    n_iter = 0
    diff = 1.0e6
    
    if ini_betas is None:
        betas = np.zeros((x.shape[1], 1), np.float)
    else:
        betas = ini_betas

    if isinstance(family, Binomial):
        y = family.link._clean(y)
    if isinstance(family, Poisson):
        y_off = y/offset
        y_off = family.starting_mu(y_off)
        v = family.predict(y_off)
        mu = family.starting_mu(y)
    else:
        mu = family.starting_mu(y)
        v = family.predict(mu)

    while diff > tol and n_iter < max_iter:
        n_iter += 1
        w = family.weights(mu)
        z = v + (family.link.deriv(mu)*(y-mu))
        w = np.sqrt(w)
        if type(x) != np.ndarray:
            w = sp.csr_matrix(w)
            z = sp.csr_matrix(z)
        wx = spmultiply(x, w, array_out=False)
        wz = spmultiply(z, w, array_out=False)
        if wi is None:
            n_betas = _compute_betas(wz, wx)
        else:
            n_betas, xtx_inv_xt = _compute_betas_gwr(wz, wx, wi)
        v = spdot(x, n_betas)
        mu  = family.fitted(v)

        if isinstance(family, Poisson):
            mu = mu * offset

        diff = min(abs(n_betas-betas))
        betas = n_betas

    if wi is None:
        return betas, mu, wx, n_iter
    else:
        return betas, mu, v, w, z, xtx_inv_xt, n_iter



import copy
from registry import user as mr, clstypes 
from pysal.weights import W
try:
    import patsy as p
except:
    p = None
from numpy import array, ndarray, asarray
from pysal.common import iteritems as diter
import inspect

#would like to just wrap this in the opt decorator...
def pandashandler(formula_like, data):
    """
    process a pysal model signature and convert an equation/formula pair into a
    pysal-specific object
    """
    if '||' in formula_like:
        mu, inst = formula_like.split('||')
        y, X = p.dmatrices(mu + '-1' , data=data)
        yend, q  = p.dmatrices(inst + '-1', data=data)
        rargs = [y,X,yend,q]
        rargs = [asarray(i) for i in rargs]
        name_y, name_x = mu.strip(' ').split('~')
        name_x = name_x.split('+')
        name_yend, name_q = inst.strip(' ').split('~')
        name_yend = [name_yend]
        name_q = name_q.split('+')
        names = {"name_y":name_y,
                 "name_x":name_x, 
                 "name_yend":name_yend,
                 "name_q":name_q}
    else:
        y, X = p.dmatrices(formula_like + '-1', data=data)
        rargs = [asarray(y), asarray(X)]
        name_y, name_x = formula_like.strip(' ').split('~')
        name_x = name_x.split('+')
        names = {"name_y":name_y,
                 "name_x":name_x}

    return rargs, names

#def model(eqn, *args, data=df, **kwargs)
class Model(object):
    """
    The model manager
    
    arguments that sit above the pysal model API:

    mtype : string mapping to the function called from spreg
    fit : Bool denoting whether to actually apply the mtype to the data provided
          immediately.

    an example call would look like:

        >>> Model(y,X,W, mtype='ML_Lag')
        >>> Model(y,X,W, mytpe='OLS_Regimes')
    """
    def __init__(self, *args, **kwargs):
        self._outers = {}
        mtype = kwargs.pop('mtype', 'OLS')
        if mtype.startswith('Base'):
            raise Exception('Only Userclasses can be fit using the handler')
        mtype = mtype
        mfunc = mr.__dict__[mtype]
        fit = kwargs.pop('fit', True)
        ins = inspect.getargspec(mfunc.__init__)
        req = len(ins.args) - (len(ins.defaults)+1 )

        if isinstance(args[0], str):
            if len(args) > 1:
                raise TypeError('When a formula is used as the first argument,'
                                ' all other arguments must be named. Consult {}'
                                ' for argument names'.format(self._mfunc.__name__))
            formula = args[0]
            data = kwargs.pop('data')
            matrices, names = pandashandler(formula, data)
            kwargs.update(names)
        elif 'formula' in kwargs.keys() and 'data' in kwargs.keys():
            formula = kwargs.pop('formula')
            data = kwargs.pop('data')
            matrices, names = pandashandler(formula, data)
            kwargs.update(names)
        else:
            matrices = list(args[0:req])

        if fit:
            self._called = mfunc(*matrices, **kwargs)
            self._fit = True
        else:
            self._outers['mtype'] = mtype
            self._outers['mfunc'] = mfunc
            self._fit = False
    
    @property
    def __dict__(self):
        inners = self._called.__dict__
        obligations = [x for x in dir(self._mfunc) if not x.startswith('_')]
        obligations = {k:self._called.__getattribute__(k) for k in obligations}
        outers = self._outers
        alldict = dict()
        alldict.update(inners)
        alldict.update(obligations)
        alldict.update(outers)
        return alldict
    
    @__dict__.setter
    def __dict__(self, key, val):
        self._outers[key] = val

    @property
    def _mfunc(self):
        return type(self._called)
    
    @property
    def _mtype(self):
        return type(self._called).__name__

    def __getattr__(self, val):
        return self._called.__getattribute__(val)

if __name__ == '__main__':
    import pysal as ps

    dbf = ps.open(ps.examples.get_path('columbus.dbf'))
    y, X = dbf.by_col_array(['HOVAL']), dbf.by_col_array(['INC', 'CRIME'])
    Wcol = ps.open(ps.examples.get_path('columbus.gal')).read()
    mod1 = mr.OLS(y,X)
    hmod1 = Model(y,X)

    mod2 = mr.OLS(y,X,w=Wcol, spat_diag=True, moran=True)
    hmod2 = Model(y,X,w=Wcol, spat_diag=True, moran=True)

    mod3 = mr.ML_Lag(y,X,Wcol)
    hmod3 = Model(y,X,Wcol, mtype='ML_Lag')

    mod4 = mr.ML_Error(y,X,Wcol)
    hmod4 = Model(y,X,w=Wcol,mtype='ML_Error')

    #real power comes from this, though
#    import geopandas as gpd
#    
#    df = gpd.read_file(ps.examples.get_path('columbus.dbf'))
#
#    hmod1_pd = Model('HOVAL ~ INC + CRIME', data=data)
#    mod5 = sr.TSLS('HOVAL ~ INC + CRIME')



"""Probit regression class and diagnostics."""

__author__ = "Luc Anselin luc.anselin@asu.edu, Pedro V. Amaral pedro.amaral@asu.edu"

import numpy as np
import numpy.linalg as la
import scipy.optimize as op
from scipy.stats import norm, chi2
chisqprob = chi2.sf
import scipy.sparse as SP
import user_output as USER
import summary_output as SUMMARY
from utils import spdot, spbroadcast

__all__ = ["Probit"]


class BaseProbit(object):

    """
    Probit class to do all the computations

    Parameters
    ----------

    x           : array
                  nxk array of independent variables (assumed to be aligned with y)
    y           : array
                  nx1 array of dependent binary variable
    w           : W
                  PySAL weights instance or spatial weights sparse matrix
                  aligned with y
    optim       : string
                  Optimization method.
                  Default: 'newton' (Newton-Raphson).
                  Alternatives: 'ncg' (Newton-CG), 'bfgs' (BFGS algorithm)
    scalem      : string
                  Method to calculate the scale of the marginal effects.
                  Default: 'phimean' (Mean of individual marginal effects)
                  Alternative: 'xmean' (Marginal effects at variables mean)
    maxiter     : int
                  Maximum number of iterations until optimizer stops                  

    Attributes
    ----------

    x           : array
                  Two dimensional array with n rows and one column for each
                  independent (exogenous) variable, including the constant
    y           : array
                  nx1 array of dependent variable
    betas       : array
                  kx1 array with estimated coefficients
    predy       : array
                  nx1 array of predicted y values
    n           : int
                  Number of observations
    k           : int
                  Number of variables
    vm          : array
                  Variance-covariance matrix (kxk)
    z_stat      : list of tuples
                  z statistic; each tuple contains the pair (statistic,
                  p-value), where each is a float                  
    xmean       : array
                  Mean of the independent variables (kx1)
    predpc      : float
                  Percent of y correctly predicted
    logl        : float
                  Log-Likelihhod of the estimation
    scalem      : string
                  Method to calculate the scale of the marginal effects.
    scale       : float
                  Scale of the marginal effects.
    slopes      : array
                  Marginal effects of the independent variables (k-1x1)
                  
		  Note: Disregards the presence of dummies.
    slopes_vm   : array
                  Variance-covariance matrix of the slopes (k-1xk-1)
    LR          : tuple
                  Likelihood Ratio test of all coefficients = 0
                  
		  (test statistics, p-value)
    Pinkse_error: float
                  Lagrange Multiplier test against spatial error correlation.
                  
		  Implemented as presented in [Pinkse2004]_              
    KP_error    : float
                  Moran's I type test against spatial error correlation.
                  
		  Implemented as presented in  [Kelejian2001]_
    PS_error    : float
                  Lagrange Multiplier test against spatial error correlation.
                  Implemented as presented in  [Pinkse1998]_
    warning     : boolean
                  if True Maximum number of iterations exceeded or gradient 
                  and/or function calls not changing.

    Examples
    --------
    >>> import numpy as np
    >>> import pysal
    >>> dbf = pysal.open(pysal.examples.get_path('columbus.dbf'),'r')
    >>> y = np.array([dbf.by_col('CRIME')]).T
    >>> x = np.array([dbf.by_col('INC'), dbf.by_col('HOVAL')]).T
    >>> x = np.hstack((np.ones(y.shape),x))
    >>> w = pysal.open(pysal.examples.get_path("columbus.gal"), 'r').read()
    >>> w.transform='r'
    >>> model = BaseProbit((y>40).astype(float), x, w=w)    
    >>> np.around(model.betas, decimals=6)
    array([[ 3.353811],
           [-0.199653],
           [-0.029514]])

    >>> np.around(model.vm, decimals=6)
    array([[ 0.852814, -0.043627, -0.008052],
           [-0.043627,  0.004114, -0.000193],
           [-0.008052, -0.000193,  0.00031 ]])

    >>> tests = np.array([['Pinkse_error','KP_error','PS_error']])
    >>> stats = np.array([[model.Pinkse_error[0],model.KP_error[0],model.PS_error[0]]])
    >>> pvalue = np.array([[model.Pinkse_error[1],model.KP_error[1],model.PS_error[1]]])
    >>> print np.hstack((tests.T,np.around(np.hstack((stats.T,pvalue.T)),6)))
    [['Pinkse_error' '3.131719' '0.076783']
     ['KP_error' '1.721312' '0.085194']
     ['PS_error' '2.558166' '0.109726']]
    """

    def __init__(self, y, x, w=None, optim='newton', scalem='phimean', maxiter=100):
        self.y = y
        self.x = x
        self.n, self.k = x.shape
        self.optim = optim
        self.scalem = scalem
        self.w = w
        self.maxiter = maxiter
        par_est, self.warning = self.par_est()
        self.betas = np.reshape(par_est[0], (self.k, 1))
        self.logl = -float(par_est[1])

    @property
    def vm(self):
        try:
            return self._cache['vm']
        except AttributeError:
            self._cache = {}
            H = self.hessian(self.betas)
            self._cache['vm'] = -la.inv(H)
        except KeyError:
            H = self.hessian(self.betas)
            self._cache['vm'] = -la.inv(H)
        return self._cache['vm']
    
    @vm.setter
    def vm(self, val):
        try:
            self._cache['vm'] = val
        except AttributeError:
            self._cache = {}
        self._cache['vm'] = val

    @property #could this get packaged into a separate function or something? It feels weird to duplicate this.  
    def z_stat(self):
        try: 
            return self._cache['z_stat']
        except AttributeError:
            self._cache = {}
            variance = self.vm.diagonal()
            zStat = self.betas.reshape(len(self.betas),) / np.sqrt(variance)
            rs = {}
            for i in range(len(self.betas)):
                rs[i] = (zStat[i], norm.sf(abs(zStat[i])) * 2)
            self._cache['z_stat'] = rs.values()
        except KeyError:
            variance = self.vm.diagonal()
            zStat = self.betas.reshape(len(self.betas),) / np.sqrt(variance)
            rs = {}
            for i in range(len(self.betas)):
                rs[i] = (zStat[i], norm.sf(abs(zStat[i])) * 2)
            self._cache['z_stat'] = rs.values()
        return self._cache['z_stat']

    @z_stat.setter
    def z_stat(self, val):
        try:
            self._cache['z_stat'] = val
        except AttributeError:
            self._cache = {}
        self._cache['z_stat'] = val

    @property
    def slopes_std_err(self):
        try:
            return self._cache['slopes_std_err']
        except AttributeError:
            self._cache = {}
            self._cache['slopes_std_err'] = np.sqrt(self.slopes_vm.diagonal())
        except KeyError:
            self._cache['slopes_std_err'] = np.sqrt(self.slopes_vm.diagonal())
        return self._cache['slopes_std_err']    
    
    @slopes_std_err.setter
    def slopes_std_err(self, val):
        try:
            self._cache['slopes_std_err'] = val
        except AttributeError:
            self._cache = {}
        self._cache['slopes_std_err'] = val

    @property
    def slopes_z_stat(self):
        try:
            return self._cache['slopes_z_stat']
        except AttributeError:
            self._cache = {}
            zStat = self.slopes.reshape(
                len(self.slopes),) / self.slopes_std_err
            rs = {}
            for i in range(len(self.slopes)):
                rs[i] = (zStat[i], norm.sf(abs(zStat[i])) * 2)
            self._cache['slopes_z_stat'] = rs.values()
        except KeyError:
            zStat = self.slopes.reshape(
                len(self.slopes),) / self.slopes_std_err
            rs = {}
            for i in range(len(self.slopes)):
                rs[i] = (zStat[i], norm.sf(abs(zStat[i])) * 2)
            self._cache['slopes_z_stat'] = rs.values()
        return self._cache['slopes_z_stat']

    @slopes_z_stat.setter
    def slopes_z_stat(self, val):
        try:
            self._cache['slopes_z_stat'] = val
        except AttributeError:
            self._cache = {}
        self._cache['slopes_z_stat'] = val

    @property
    def xmean(self):
        try:
            return self._cache['xmean']
        except AttributeError:
            self._cache = {}
            try: #why is this try-accept? can x be a list??
                self._cache['xmean'] = np.reshape(sum(self.x) / self.n, (self.k, 1))
            except:
                self._cache['xmean'] = np.reshape(sum(self.x).toarray() / self.n, (self.k, 1))
        except KeyError:
            try:
                self._cache['xmean'] = np.reshape(sum(self.x) / self.n, (self.k, 1))
            except:
                self._cache['xmean'] = np.reshape(sum(self.x).toarray() / self.n, (self.k, 1))
        return self._cache['xmean']

    @xmean.setter
    def xmean(self, val):
        try:
            self._cache['xmean'] = val
        except AttributeError:
            self._cache = {}
        self._cache['xmean'] = val

    @property
    def xb(self):
        try:
            return self._cache['xb']
        except AttributeError:
            self._cache = {}
            self._cache['xb'] = spdot(self.x, self.betas)
        except KeyError:
            self._cache['xb'] = spdot(self.x, self.betas)
        return self._cache['xb']

    @xb.setter
    def xb(self, val):
        try:
            self._cache['xb'] = val
        except AttributeError:
            self._cache = {}
        self._cache['xb'] = val

    @property
    def predy(self):
        try:
            return self._cache['predy']
        except AttributeError:
            self._cache = {}
            self._cache['predy'] = norm.cdf(self.xb)
        except KeyError:
            self._cache['predy'] = norm.cdf(self.xb)
        return self._cache['predy']
    
    @predy.setter
    def predy(self, val):
        try:
            self._cache['predy'] = val
        except AttributeError:
            self._cache = {}
        self._cache['predy'] = val

    @property
    def predpc(self):
        try:
            return self._cache['predpc']
        except AttributeError:
            self._cache = {}
            predpc = abs(self.y - self.predy)
            for i in range(len(predpc)):
                if predpc[i] > 0.5:
                    predpc[i] = 0
                else:
                    predpc[i] = 1
            self._cache['predpc'] = float(100.0 * np.sum(predpc) / self.n)
        except KeyError:
            predpc = abs(self.y - self.predy)
            for i in range(len(predpc)):
                if predpc[i] > 0.5:
                    predpc[i] = 0
                else:
                    predpc[i] = 1
            self._cache['predpc'] = float(100.0 * np.sum(predpc) / self.n)
        return self._cache['predpc']

    @predpc.setter
    def predpc(self, val):
        try:
            self._cache['predpc'] = val
        except AttributeError:
            self._cache = {}
        self._cache['predpc'] = val
    
    @property
    def phiy(self):
        try:
            return self._cache['phiy']
        except AttributeError:
            self._cache = {}
            self._cache['phiy'] = norm.pdf(self.xb)
        except KeyError:
            self._cache['phiy'] = norm.pdf(self.xb)
        return self._cache['phiy']
    
    @phiy.setter
    def phiy(self, val):
        try:
            self._cache['phiy'] = val
        except AttributeError:
            self._cache = {}
        self._cache['phiy'] = val

    @property
    def scale(self):
        try:
            return self._cache['scale']
        except AttributeError:
            self._cache = {}
            if self.scalem == 'phimean':
                self._cache['scale'] = float(1.0 * np.sum(self.phiy) / self.n)
            elif self.scalem == 'xmean':
                self._cache['scale'] = float(norm.pdf(np.dot(self.xmean.T, self.betas)))
        except KeyError:
            if self.scalem == 'phimean':
                self._cache['scale'] = float(1.0 * np.sum(self.phiy) / self.n)
            if self.scalem == 'xmean':
                self._cache['scale'] = float(norm.pdf(np.dot(self.xmean.T, self.betas)))
        return self._cache['scale']

    @scale.setter
    def scale(self, val):
        try:
            self._cache['scale'] = val
        except AttributeError:
            self._cache = {}
        self._cache['scale'] = val

    @property
    def slopes(self):
        try:
            return self._cache['slopes']
        except AttributeError:
            self._cache = {}
            self._cache['slopes'] = self.betas[1:] * self.scale
        except KeyError:
            self._cache['slopes'] = self.betas[1:] * self.scale
        return self._cache['slopes']

    @slopes.setter
    def slopes(self, val):
        try:
            self._cache['slopes'] = val
        except AttributeError:
            self._cache = {}
        self._cache['slopes'] = val

    @property
    def slopes_vm(self):
        try:
            return self._cache['slopes_vm']
        except AttributeError:
            self._cache = {}
            x = self.xmean
            b = self.betas
            dfdb = np.eye(self.k) - spdot(b.T, x) * spdot(b, x.T)
            slopes_vm = (self.scale ** 2) * \
                np.dot(np.dot(dfdb, self.vm), dfdb.T)
            self._cache['slopes_vm'] = slopes_vm[1:, 1:]
        except KeyError:
            x = self.xmean
            b = self.betas
            dfdb = np.eye(self.k) - spdot(b.T, x) * spdot(b, x.T)
            slopes_vm = (self.scale ** 2) * \
                np.dot(np.dot(dfdb, self.vm), dfdb.T)
            self._cache['slopes_vm'] = slopes_vm[1:, 1:]
        return self._cache['slopes_vm']

    @slopes_vm.setter
    def slopes_vm(self, val):
        try:
            self._cache['slopes_vm'] = val
        except AttributeError:
            self._cache = {}
        self._cache['slopes_vm'] = val

    @property
    def LR(self):
        try:
            return self._cache['LR']
        except AttributeError:
            self._cache = {}
            P = 1.0 * np.sum(self.y) / self.n
            LR = float(
                -2 * (self.n * (P * np.log(P) + (1 - P) * np.log(1 - P)) - self.logl))
            self._cache['LR'] = (LR, chisqprob(LR, self.k))
        except KeyError:
            P = 1.0 * np.sum(self.y) / self.n
            LR = float(
                -2 * (self.n * (P * np.log(P) + (1 - P) * np.log(1 - P)) - self.logl))
            self._cache['LR'] = (LR, chisqprob(LR, self.k))
        return self._cache['LR']

    @LR.setter
    def LR(self, val):
        try:
            self._cache['LR'] = val
        except AttributeError:
            self._cache = {}
        self._cache['LR'] = val

    @property
    def u_naive(self):
        try:
            return self._cache['u_naive']
        except AttributeError:
            self._cache = {}
            self._cache['u_naive'] = self.y - self.predy
        except KeyError:
            u_naive = self.y - self.predy
            self._cache['u_naive'] = u_naive
        return self._cache['u_naive']

    @u_naive.setter
    def u_naive(self, val):
        try:
            self._cache['u_naive'] = val
        except AttributeError:
            self._cache = {}
        self._cache['u_naive'] = val
    
    @property
    def u_gen(self):
        try:
            return self._cache['u_gen']
        except AttributeError:
            self._cache = {}
            Phi_prod = self.predy * (1 - self.predy)
            u_gen = self.phiy * (self.u_naive / Phi_prod)
            self._cache['u_gen'] = u_gen
        except KeyError:
            Phi_prod = self.predy * (1 - self.predy)
            u_gen = self.phiy * (self.u_naive / Phi_prod)
            self._cache['u_gen'] = u_gen
        return self._cache['u_gen']
    
    @u_gen.setter
    def u_gen(self, val):
        try:
            self._cache['u_gen'] = val
        except AttributeError:
            self._cache = {}
        self._cache['u_gen'] = val

    @property
    def Pinkse_error(self):
        try:
            return self._cache['Pinkse_error']
        except AttributeError:
            self._cache = {}
            self._cache['Pinkse_error'], self._cache[
                'KP_error'], self._cache['PS_error'] = sp_tests(self)
        except KeyError:
            self._cache['Pinkse_error'], self._cache[
                'KP_error'], self._cache['PS_error'] = sp_tests(self)
        return self._cache['Pinkse_error']

    @Pinkse_error.setter
    def Pinkse_error(self, val):
        try:
            self._cache['Pinkse_error'] = val
        except AttributeError:
            self._cache = {}
        self._cache['Pinkse_error'] = val

    @property
    def KP_error(self):
        try:
            return self._cache['KP_error']
        except AttributeError:
            self._cache = {}
            self._cache['Pinkse_error'], self._cache[
                'KP_error'], self._cache['PS_error'] = sp_tests(self)
        except KeyError:
            self._cache['Pinkse_error'], self._cache[
                'KP_error'], self._cache['PS_error'] = sp_tests(self)
        return self._cache['KP_error']

    @KP_error.setter
    def KP_error(self, val):
        try:
            self._cache['KP_error'] = val
        except AttributeError:
            self._cache = {}
        self._cache['KP_error'] = val

    @property
    def PS_error(self):
        try:
            return self._cache['PS_error']
        except AttributeError:
            self._cache = {}
            self._cache['Pinkse_error'], self._cache[
                'KP_error'], self._cache['PS_error'] = sp_tests(self)
        except KeyError:
            self._cache['Pinkse_error'], self._cache[
                'KP_error'], self._cache['PS_error'] = sp_tests(self)
        return self._cache['PS_error']

    @PS_error.setter
    def PS_error(self, val):
        try:
            self._cache['PS_error'] = val
        except AttributeError:
            self._cache = {}
        self._cache['PS_error'] = val

    def par_est(self):
        start = np.dot(la.inv(spdot(self.x.T, self.x)),
                       spdot(self.x.T, self.y))
        flogl = lambda par: -self.ll(par)
        if self.optim == 'newton':
            fgrad = lambda par: self.gradient(par)
            fhess = lambda par: self.hessian(par)
            par_hat = newton(flogl, start, fgrad, fhess, self.maxiter)
            warn = par_hat[2]
        else:
            fgrad = lambda par: -self.gradient(par)
            if self.optim == 'bfgs':
                par_hat = op.fmin_bfgs(
                    flogl, start, fgrad, full_output=1, disp=0)
                warn = par_hat[6]
            if self.optim == 'ncg':
                fhess = lambda par: -self.hessian(par)
                par_hat = op.fmin_ncg(
                    flogl, start, fgrad, fhess=fhess, full_output=1, disp=0)
                warn = par_hat[5]
        if warn > 0:
            warn = True
        else:
            warn = False
        return par_hat, warn

    def ll(self, par):
        beta = np.reshape(np.array(par), (self.k, 1))
        q = 2 * self.y - 1
        qxb = q * spdot(self.x, beta)
        ll = sum(np.log(norm.cdf(qxb)))
        return ll

    def gradient(self, par):
        beta = np.reshape(np.array(par), (self.k, 1))
        q = 2 * self.y - 1
        qxb = q * spdot(self.x, beta)
        lamb = q * norm.pdf(qxb) / norm.cdf(qxb)
        gradient = spdot(lamb.T, self.x)[0]
        return gradient

    def hessian(self, par):
        beta = np.reshape(np.array(par), (self.k, 1))
        q = 2 * self.y - 1
        xb = spdot(self.x, beta)
        qxb = q * xb
        lamb = q * norm.pdf(qxb) / norm.cdf(qxb)
        hessian = spdot(self.x.T, spbroadcast(self.x,-lamb * (lamb + xb)))
        return hessian


class Probit(BaseProbit):

    """
    Classic non-spatial Probit and spatial diagnostics. The class includes a
    printout that formats all the results and tests in a nice format.

    The diagnostics for spatial dependence currently implemented are:

        * Pinkse Error [Pinkse2004]_
        * Kelejian and Prucha Moran's I [Kelejian2001]_
        * Pinkse & Slade Error [Pinkse1998]_

    Parameters
    ----------

    x           : array
                  nxk array of independent variables (assumed to be aligned with y)
    y           : array
                  nx1 array of dependent binary variable
    w           : W
                  PySAL weights instance aligned with y
    optim       : string
                  Optimization method.
                  Default: 'newton' (Newton-Raphson).
                  Alternatives: 'ncg' (Newton-CG), 'bfgs' (BFGS algorithm)
    scalem      : string
                  Method to calculate the scale of the marginal effects.
                  Default: 'phimean' (Mean of individual marginal effects)
                  Alternative: 'xmean' (Marginal effects at variables mean)
    maxiter     : int
                  Maximum number of iterations until optimizer stops                  
    name_y       : string
                   Name of dependent variable for use in output
    name_x       : list of strings
                   Names of independent variables for use in output
    name_w       : string
                   Name of weights matrix for use in output
    name_ds      : string
                   Name of dataset for use in output

    Attributes
    ----------

    x           : array
                  Two dimensional array with n rows and one column for each
                  independent (exogenous) variable, including the constant
    y           : array
                  nx1 array of dependent variable
    betas       : array
                  kx1 array with estimated coefficients
    predy       : array
                  nx1 array of predicted y values
    n           : int
                  Number of observations
    k           : int
                  Number of variables
    vm          : array
                  Variance-covariance matrix (kxk)
    z_stat      : list of tuples
                  z statistic; each tuple contains the pair (statistic,
                  p-value), where each is a float                  
    xmean       : array
                  Mean of the independent variables (kx1)
    predpc      : float
                  Percent of y correctly predicted
    logl        : float
                  Log-Likelihhod of the estimation
    scalem      : string
                  Method to calculate the scale of the marginal effects.
    scale       : float
                  Scale of the marginal effects.
    slopes      : array
                  Marginal effects of the independent variables (k-1x1)
    slopes_vm   : array
                  Variance-covariance matrix of the slopes (k-1xk-1)
    LR          : tuple
                  Likelihood Ratio test of all coefficients = 0
                  (test statistics, p-value)
    Pinkse_error: float
                  Lagrange Multiplier test against spatial error correlation.
                  Implemented as presented in  [Pinkse2004]_             
    KP_error    : float
                  Moran's I type test against spatial error correlation.
                  Implemented as presented in [Kelejian2001]_
    PS_error    : float
                  Lagrange Multiplier test against spatial error correlation.
                  Implemented as presented in [Pinkse1998]_
    warning     : boolean
                  if True Maximum number of iterations exceeded or gradient 
                  and/or function calls not changing.
    name_y       : string
                   Name of dependent variable for use in output
    name_x       : list of strings
                   Names of independent variables for use in output
    name_w       : string
                   Name of weights matrix for use in output
    name_ds      : string
                   Name of dataset for use in output
    title        : string
                   Name of the regression method used

    Examples
    --------

    We first need to import the needed modules, namely numpy to convert the
    data we read into arrays that ``spreg`` understands and ``pysal`` to
    perform all the analysis.

    >>> import numpy as np
    >>> import pysal

    Open data on Columbus neighborhood crime (49 areas) using pysal.open().
    This is the DBF associated with the Columbus shapefile.  Note that
    pysal.open() also reads data in CSV format; since the actual class
    requires data to be passed in as numpy arrays, the user can read their
    data in using any method.  

    >>> dbf = pysal.open(pysal.examples.get_path('columbus.dbf'),'r')

    Extract the CRIME column (crime) from the DBF file and make it the
    dependent variable for the regression. Note that PySAL requires this to be
    an numpy array of shape (n, 1) as opposed to the also common shape of (n, )
    that other packages accept. Since we want to run a probit model and for this
    example we use the Columbus data, we also need to transform the continuous
    CRIME variable into a binary variable. As in [McMillen1992]_, we define
    y = 1 if CRIME > 40.

    >>> y = np.array([dbf.by_col('CRIME')]).T
    >>> y = (y>40).astype(float)

    Extract HOVAL (home values) and INC (income) vectors from the DBF to be used as
    independent variables in the regression.  Note that PySAL requires this to
    be an nxj numpy array, where j is the number of independent variables (not
    including a constant). By default this class adds a vector of ones to the
    independent variables passed in.

    >>> names_to_extract = ['INC', 'HOVAL']
    >>> x = np.array([dbf.by_col(name) for name in names_to_extract]).T

    Since we want to the test the probit model for spatial dependence, we need to
    specify the spatial weights matrix that includes the spatial configuration of
    the observations into the error component of the model. To do that, we can open
    an already existing gal file or create a new one. In this case, we will use
    ``columbus.gal``, which contains contiguity relationships between the
    observations in the Columbus dataset we are using throughout this example.
    Note that, in order to read the file, not only to open it, we need to
    append '.read()' at the end of the command.

    >>> w = pysal.open(pysal.examples.get_path("columbus.gal"), 'r').read() 

    Unless there is a good reason not to do it, the weights have to be
    row-standardized so every row of the matrix sums to one. In PySAL, this
    can be easily performed in the following way:

    >>> w.transform='r'

    We are all set with the preliminaries, we are good to run the model. In this
    case, we will need the variables and the weights matrix. If we want to
    have the names of the variables printed in the output summary, we will
    have to pass them in as well, although this is optional. 

    >>> model = Probit(y, x, w=w, name_y='crime', name_x=['income','home value'], name_ds='columbus', name_w='columbus.gal')

    Once we have run the model, we can explore a little bit the output. The
    regression object we have created has many attributes so take your time to
    discover them.

    >>> np.around(model.betas, decimals=6)
    array([[ 3.353811],
           [-0.199653],
           [-0.029514]])

    >>> np.around(model.vm, decimals=6)
    array([[ 0.852814, -0.043627, -0.008052],
           [-0.043627,  0.004114, -0.000193],
           [-0.008052, -0.000193,  0.00031 ]])

    Since we have provided a spatial weigths matrix, the diagnostics for
    spatial dependence have also been computed. We can access them and their
    p-values individually:

    >>> tests = np.array([['Pinkse_error','KP_error','PS_error']])
    >>> stats = np.array([[model.Pinkse_error[0],model.KP_error[0],model.PS_error[0]]])
    >>> pvalue = np.array([[model.Pinkse_error[1],model.KP_error[1],model.PS_error[1]]])
    >>> print np.hstack((tests.T,np.around(np.hstack((stats.T,pvalue.T)),6)))
    [['Pinkse_error' '3.131719' '0.076783']
     ['KP_error' '1.721312' '0.085194']
     ['PS_error' '2.558166' '0.109726']]

    Or we can easily obtain a full summary of all the results nicely formatted and
    ready to be printed simply by typing 'print model.summary'

    """

    def __init__(
        self, y, x, w=None, optim='newton', scalem='phimean', maxiter=100,
        vm=False, name_y=None, name_x=None, name_w=None, name_ds=None,
            spat_diag=False):

        n = USER.check_arrays(y, x)
        USER.check_y(y, n)
        if w != None:
            USER.check_weights(w, y)
            spat_diag = True
            ws = w.sparse
        else:
            ws = None
        x_constant = USER.check_constant(x)
        BaseProbit.__init__(self, y=y, x=x_constant, w=ws,
                            optim=optim, scalem=scalem, maxiter=maxiter)
        self.title = "CLASSIC PROBIT ESTIMATOR"
        self.name_ds = USER.set_name_ds(name_ds)
        self.name_y = USER.set_name_y(name_y)
        self.name_x = USER.set_name_x(name_x, x)
        self.name_w = USER.set_name_w(name_w, w)
        SUMMARY.Probit(reg=self, w=w, vm=vm, spat_diag=spat_diag)


def newton(flogl, start, fgrad, fhess, maxiter):
    """
    Calculates the Newton-Raphson method

    Parameters
    ----------

    flogl       : lambda
                  Function to calculate the log-likelihood
    start       : array
                  kx1 array of starting values
    fgrad       : lambda
                  Function to calculate the gradient
    fhess       : lambda
                  Function to calculate the hessian
    maxiter     : int
                  Maximum number of iterations until optimizer stops                
    """
    warn = 0
    iteration = 0
    par_hat0 = start
    m = 1
    while (iteration < maxiter and m >= 1e-04):
        H = -la.inv(fhess(par_hat0))
        g = fgrad(par_hat0).reshape(start.shape)
        Hg = np.dot(H, g)
        par_hat0 = par_hat0 + Hg
        iteration += 1
        m = np.dot(g.T, Hg)
    if iteration == maxiter:
        warn = 1
    logl = flogl(par_hat0)
    return (par_hat0, logl, warn)


def sp_tests(reg):
    """
    Calculates tests for spatial dependence in Probit models

    Parameters
    ----------

    reg         : regression object
                  output instance from a probit model            
    """
    if reg.w != None:
        try:
            w = reg.w.sparse
        except:
            w = reg.w
        Phi = reg.predy
        phi = reg.phiy
        # Pinkse_error:
        Phi_prod = Phi * (1 - Phi)
        u_naive = reg.u_naive
        u_gen = reg.u_gen
        sig2 = np.sum((phi * phi) / Phi_prod) / reg.n
        LM_err_num = np.dot(u_gen.T, (w * u_gen)) ** 2
        trWW = np.sum((w * w).diagonal())
        trWWWWp = trWW + np.sum((w * w.T).diagonal())
        LM_err = float(1.0 * LM_err_num / (sig2 ** 2 * trWWWWp))
        LM_err = np.array([LM_err, chisqprob(LM_err, 1)])
        # KP_error:
        moran = moran_KP(reg.w, u_naive, Phi_prod)
        # Pinkse-Slade_error:
        u_std = u_naive / np.sqrt(Phi_prod)
        ps_num = np.dot(u_std.T, (w * u_std)) ** 2
        trWpW = np.sum((w.T * w).diagonal())
        ps = float(ps_num / (trWW + trWpW))
        # chi-square instead of bootstrap.
        ps = np.array([ps, chisqprob(ps, 1)])
    else:
        raise Exception, "W matrix must be provided to calculate spatial tests."
    return LM_err, moran, ps


def moran_KP(w, u, sig2i):
    """
    Calculates Moran-flavoured tests 

    Parameters
    ----------

    w           : W
                  PySAL weights instance aligned with y
    u           : array
                  nx1 array of naive residuals
    sig2i       : array
                  nx1 array of individual variance               
    """
    try:
        w = w.sparse
    except:
        pass
    moran_num = np.dot(u.T, (w * u))
    E = SP.lil_matrix(w.get_shape())
    E.setdiag(sig2i.flat)
    E = E.asformat('csr')
    WE = w * E
    moran_den = np.sqrt(np.sum((WE * WE + (w.T * E) * WE).diagonal()))
    moran = float(1.0 * moran_num / moran_den)
    moran = np.array([moran, norm.sf(abs(moran)) * 2.])
    return moran


def _test():
    import doctest
    start_suppress = np.get_printoptions()['suppress']
    np.set_printoptions(suppress=True)
    doctest.testmod()
    np.set_printoptions(suppress=start_suppress)

if __name__ == '__main__':
    _test()
    import numpy as np
    import pysal
    dbf = pysal.open(pysal.examples.get_path('columbus.dbf'), 'r')
    y = np.array([dbf.by_col('CRIME')]).T
    var_x = ['INC', 'HOVAL']
    x = np.array([dbf.by_col(name) for name in var_x]).T
    w = pysal.open(pysal.examples.get_path("columbus.gal"), 'r').read()
    w.transform = 'r'
    probit1 = Probit(
        (y > 40).astype(float), x, w=w, name_x=var_x, name_y="CRIME",
        name_ds="Columbus", name_w="columbus.dbf")
    print probit1.summary



import pysal.core.Tables as Tables

__author__ = "Charles R Schmidt <schmidtc@gmail.com>"
__all__ = ['GeoDaTxtReader']


class GeoDaTxtReader(Tables.DataTable):
    """GeoDa Text File Export Format
    """
    __doc__ = Tables.DataTable.__doc__
    FORMATS = ['geoda_txt']
    MODES = ['r']

    def __init__(self, *args, **kwargs):
        """
        Examples
        --------
        >>> import pysal
        >>> f = pysal.open(pysal.examples.get_path('stl_hom.txt'),'r')
        >>> f.header
        ['FIPSNO', 'HR8488', 'HR8893', 'HC8488']
        >>> len(f)
        78
        >>> f.dat[0]
        ['17107', '1.290722', '1.624458', '2']
        >>> f.dat[-1]
        ['29223', '0', '8.451537', '0']
        >>> f._spec
        [<type 'int'>, <type 'float'>, <type 'float'>, <type 'int'>]

        """
        Tables.DataTable.__init__(self, *args, **kwargs)
        self.__idx = {}
        self.__len = None
        self.pos = 0
        self._open()

    def _open(self):
        if self.mode == 'r':
            self.fileObj = open(self.dataPath, 'r')
            n, k = self.fileObj.readline().strip().split(',')
            n, k = int(n), int(k)
            header = self.fileObj.readline().strip().split(',')
            self.header = [f.replace('"', '') for f in header]
            try:
                assert len(self.header) == k
            except AssertionError:
                raise TypeError("This is not a valid geoda_txt file.")
            dat = self.fileObj.readlines()
            self.dat = [line.strip().split(',') for line in dat]
            self._spec = self._determineSpec(self.dat)
            self.__len = len(dat)

    def __len__(self):
        return self.__len

    def _read(self):
        if self.pos < len(self):
            row = self.dat[self.pos]
            self.pos += 1
            return row
        else:
            return None

    def close(self):
        self.fileObj.close()
        Tables.DataTable.close(self)

    @staticmethod
    def _determineSpec(data):
        cols = len(data[0])
        spec = []
        for j in range(cols):
            isInt = True
            isFloat = True
            for row in data:
                val = row[j]
                if not val.strip().replace('-', '').replace('.', '').isdigit():
                    isInt = False
                    isFloat = False
                    break
                else:
                    if isInt and '.' in val:
                        isInt = False
            if isInt:
                spec.append(int)
            elif isFloat:
                spec.append(float)
            else:
                spec.append(str)
        return spec
